# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .element import Attribute
from .method import Method
from .printer import Printer

from typing import Iterator


class Event(Method):
    """Scanner for event objects (server-side method)

    Required attributes: `name`

    Optional attributes: `since`

    Child elements: `description` and `arg``
    """
    method_type = 'event'

    attributes = [
        Attribute('name', True),
        Attribute('since', False)
    ]

    @property
    def method_args(self):
        """Generator of the arguments to the method

        All arguments to be sent to `._post_event` must be passed in
        """
        for arg in self.arg:
            yield arg.name

    @property
    def interface_types(self) -> Iterator[str]:
        """Generator of the types (for the wl_interface)"""
        for arg in self.arg:
            if arg.interface:  # type: ignore
                yield arg.interface_class
            else:
                yield 'None'

    def output_doc_params(self, printer: Printer) -> None:
        """Aguments documented as parameters

        All arguments are event parameters.
        """
        for arg in self.arg:
            arg.output_doc_param(printer)

    def output_doc_ret(self, printer: Printer) -> None:
        """Aguments documented as return values

        Nothing is returned from event calls.
        """
        return

    def output_body(self, printer: Printer) -> None:
        """Output the body of the event to the printer"""
        args = ', '.join([str(self.opcode)] + list(self.method_args))
        printer('self._post_event({})'.format(args))



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .argument import Argument
from .description import Description
from .element import Child, Element

from typing import List  # noqa: F401

# For 'new_id' types with no 'interface'
NO_IFACE = 'interface'
NO_IFACE_VERSION = 'version'


class Method(Element):
    """Scanner for methods

    Corresponds to event and requests defined on an interface
    """

    children = [
        Child('description', Description, False, False),
        Child('arg', Argument, False, True),
    ]

    description = []  # type: List[Description]
    arg = []  # type: List[Argument]

    def __init__(self, method, iface_name, opcode):
        super(Method, self).__init__(method)

        self.opcode = opcode
        self.interface = iface_name

        # some methods are protected names, so append '_'
        if self.name in ('global', 'import'):
            self.name += '_'

    def imports(self, module_imports):
        """Get the imports required for each of the interfaces"""
        current_protocol = module_imports[self.interface]

        imports = []
        for arg in self.arg:
            if arg.interface is None:
                continue

            if arg.interface == self.interface:
                continue

            import_class = arg.interface_class
            import_protocol = module_imports[arg.interface]

            if current_protocol == import_protocol:
                import_path = ".{}".format(arg.interface)
            else:
                import_path = "..{}".format(import_protocol)

            imports.append((import_path, import_class))

        return imports

    def output(self, printer, in_class, module_imports):
        """Generate the output for the given method to the printer"""
        # Generate the decorator for the method
        signature = ''.join(arg.signature for arg in self.arg)
        if self.since and int(self.since) > 1:
            signature = self.since + signature
        types = ', '.join(self.interface_types)

        printer('@{}.{}("{}", [{}])'.format(
            in_class, self.method_type, signature, types)
        )

        # Generate the definition of the method and args
        args = ', '.join(['self'] + list(self.method_args))
        printer('def {}({}):'.format(self.name, args))

        with printer.indented():
            # Write the documentation
            self.output_doc(printer)
            # Write out the body of the method
            self.output_body(printer)

    def output_doc(self, printer):
        """Output the documentation for the interface"""
        if self.description:
            self.description.output(printer)
        else:
            printer('"""' + self.name)
        # Parameter and returns documentation
        if self.arg:
            printer()
            self.output_doc_params(printer)
        printer('"""')



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_surface import WlSurface

import enum


class WlKeyboard(Interface):
    """Keyboard input device

    The :class:`WlKeyboard` interface represents one or more keyboards
    associated with a seat.
    """
    name = "wl_keyboard"
    version = 7

    keymap_format = enum.Enum("keymap_format", {
        "no_keymap": 0,
        "xkb_v1": 1,
    })

    key_state = enum.Enum("key_state", {
        "released": 0,
        "pressed": 1,
    })


@WlKeyboard.request("3", [])
def release(self):
    """Release the keyboard object
    """
    self._marshal(0)
    self._destroy()


@WlKeyboard.event("uhu", [None, None, None])
def keymap(self, format, fd, size):
    """Keyboard mapping

    This event provides a file descriptor to the client which can be memory-
    mapped to provide a keyboard mapping description.

    From version 7 onwards, the fd must be mapped with MAP_PRIVATE by the
    recipient, as MAP_SHARED may fail.

    :param format:
        keymap format
    :type format: `uint`
    :param fd:
        keymap file descriptor
    :type fd: `fd`
    :param size:
        keymap size, in bytes
    :type size: `uint`
    """
    self._post_event(0, format, fd, size)


@WlKeyboard.event("uoa", [None, WlSurface, None])
def enter(self, serial, surface, keys):
    """Enter event

    Notification that this seat's keyboard focus is on a certain surface.

    :param serial:
        serial number of the enter event
    :type serial: `uint`
    :param surface:
        surface gaining keyboard focus
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param keys:
        the currently pressed keys
    :type keys: `array`
    """
    self._post_event(1, serial, surface, keys)


@WlKeyboard.event("uo", [None, WlSurface])
def leave(self, serial, surface):
    """Leave event

    Notification that this seat's keyboard focus is no longer on a certain
    surface.

    The leave notification is sent before the enter notification for the new
    focus.

    :param serial:
        serial number of the leave event
    :type serial: `uint`
    :param surface:
        surface that lost keyboard focus
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    """
    self._post_event(2, serial, surface)


@WlKeyboard.event("uuuu", [None, None, None, None])
def key(self, serial, time, key, state):
    """Key event

    A key was pressed or released. The time argument is a timestamp with
    millisecond granularity, with an undefined base.

    :param serial:
        serial number of the key event
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param key:
        key that produced the event
    :type key: `uint`
    :param state:
        physical state of the key
    :type state: `uint`
    """
    self._post_event(3, serial, time, key, state)


@WlKeyboard.event("uuuuu", [None, None, None, None, None])
def modifiers(self, serial, mods_depressed, mods_latched, mods_locked, group):
    """Modifier and group state

    Notifies clients that the modifier and/or group state has changed, and it
    should update its local state.

    :param serial:
        serial number of the modifiers event
    :type serial: `uint`
    :param mods_depressed:
        depressed modifiers
    :type mods_depressed: `uint`
    :param mods_latched:
        latched modifiers
    :type mods_latched: `uint`
    :param mods_locked:
        locked modifiers
    :type mods_locked: `uint`
    :param group:
        keyboard layout
    :type group: `uint`
    """
    self._post_event(4, serial, mods_depressed, mods_latched, mods_locked, group)


@WlKeyboard.event("4ii", [None, None])
def repeat_info(self, rate, delay):
    """Repeat rate and delay

    Informs the client about the keyboard's repeat rate and delay.

    This event is sent as soon as the :class:`WlKeyboard` object has been
    created, and is guaranteed to be received by the client before any key
    press event.

    Negative values for either rate or delay are illegal. A rate of zero will
    disable any repeating (regardless of the value of delay).

    This event can be sent later on as well with a new value if necessary, so
    clients should continue listening for the event past the creation of
    :class:`WlKeyboard`.

    :param rate:
        the rate of repeating keys in characters per second
    :type rate: `int`
    :param delay:
        delay in milliseconds since key down until repeating starts
    :type delay: `int`
    """
    self._post_event(5, rate, delay)


WlKeyboard._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface

import enum


class ZxdgToplevelDecorationV1(Interface):
    """Decoration object for a toplevel surface

    The decoration object allows the compositor to toggle server-side window
    decorations for a toplevel surface. The client can request to switch to
    another mode.

    The xdg_toplevel_decoration object must be destroyed before its
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.
    """
    name = "zxdg_toplevel_decoration_v1"
    version = 1

    error = enum.Enum("error", {
        "unconfigured_buffer": 0,
        "already_constructed": 1,
        "orphaned": 2,
    })

    mode = enum.Enum("mode", {
        "client_side": 1,
        "server_side": 2,
    })


@ZxdgToplevelDecorationV1.request("", [])
def destroy(self):
    """Destroy the decoration object

    Switch back to a mode without any server-side decorations at the next
    commit.
    """
    self._marshal(0)
    self._destroy()


@ZxdgToplevelDecorationV1.request("u", [None])
def set_mode(self, mode):
    """Set the decoration mode

    Set the toplevel surface decoration mode. This informs the compositor that
    the client prefers the provided decoration mode.

    After requesting a decoration mode, the compositor will respond by emitting
    a :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` event. The client
    should then update its content, drawing it without decorations if the
    received mode is server-side decorations. The client must also acknowledge
    the configure when committing the new content (see
    :func:`XdgSurface.ack_configure()
    <pywayland.protocol.xdg_shell.XdgSurface..ack_configure>`).

    The compositor can decide not to use the client's mode and enforce a
    different mode instead.

    Clients whose decoration mode depend on the
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` state may send a
    set_mode request in response to a :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` event and wait for
    the next :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` event to prevent
    unwanted state. Such clients are responsible for preventing configure loops
    and must make sure not to send multiple successive set_mode requests with
    the same decoration mode.

    :param mode:
        the decoration mode
    :type mode: `uint`
    """
    self._marshal(1, mode)


@ZxdgToplevelDecorationV1.request("", [])
def unset_mode(self):
    """Unset the decoration mode

    Unset the toplevel surface decoration mode. This informs the compositor
    that the client doesn't prefer a particular decoration mode.

    This request has the same semantics as set_mode.
    """
    self._marshal(2)


@ZxdgToplevelDecorationV1.event("u", [None])
def configure(self, mode):
    """Suggest a surface change

    The configure event asks the client to change its decoration mode. The
    configured state should not be applied immediately. Clients must send an
    ack_configure in response to this event. See :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` and
    :func:`XdgSurface.ack_configure()
    <pywayland.protocol.xdg_shell.XdgSurface..ack_configure>` for details.

    A configure event can be sent at any time. The specified mode must be
    obeyed by the client.

    :param mode:
        the decoration mode
    :type mode: `uint`
    """
    self._post_event(0, mode)


ZxdgToplevelDecorationV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface


class ZwpTabletV1(Interface):
    """Graphics tablet device

    The wp_tablet interface represents one graphics tablet device. The tablet
    interface itself does not generate events; all events are generated by
    wp_tablet_tool objects when in proximity above a tablet.

    A tablet has a number of static characteristics, e.g. device name and
    pid/vid. These capabilities are sent in an event sequence after the
    wp_tablet_seat.tablet_added event. This initial event sequence is
    terminated by a wp_tablet.done event.
    """
    name = "zwp_tablet_v1"
    version = 1


@ZwpTabletV1.request("", [])
def destroy(self):
    """Destroy the tablet object

    This destroys the client's resource for this tablet object.
    """
    self._marshal(0)
    self._destroy()


@ZwpTabletV1.event("s", [None])
def name(self, name):
    """Tablet device name

    This event is sent in the initial burst of events before the wp_tablet.done
    event.

    :param name:
        the device name
    :type name: `string`
    """
    self._post_event(0, name)


@ZwpTabletV1.event("uu", [None, None])
def id(self, vid, pid):
    """Tablet device usb vendor/product id

    This event is sent in the initial burst of events before the wp_tablet.done
    event.

    :param vid:
        USB vendor id
    :type vid: `uint`
    :param pid:
        USB product id
    :type pid: `uint`
    """
    self._post_event(1, vid, pid)


@ZwpTabletV1.event("s", [None])
def path(self, path):
    """Path to the device

    A system-specific device path that indicates which device is behind this
    wp_tablet. This information may be used to gather additional information
    about the device, e.g. through libwacom.

    A device may have more than one device path. If so, multiple wp_tablet.path
    events are sent. A device may be emulated and not have a device path, and
    in that case this event will not be sent.

    The format of the path is unspecified, it may be a device node, a sysfs
    path, or some other identifier. It is up to the client to identify the
    string provided.

    This event is sent in the initial burst of events before the wp_tablet.done
    event.

    :param path:
        path to local device
    :type path: `string`
    """
    self._post_event(2, path)


@ZwpTabletV1.event("", [])
def done(self):
    """Tablet description events sequence complete

    This event is sent immediately to signal the end of the initial burst of
    descriptive events. A client may consider the static description of the
    tablet to be complete and finalize initialization of the tablet.
    """
    self._post_event(3)


@ZwpTabletV1.event("", [])
def removed(self):
    """Tablet removed event

    Sent when the tablet has been removed from the system. When a tablet is
    removed, some tools may be removed.

    When this event is received, the client must wp_tablet.destroy the object.
    """
    self._post_event(4)


ZwpTabletV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2013-2016 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSurface
from .wp_viewport import WpViewport

import enum


class WpViewporter(Interface):
    """Surface cropping and scaling

    The global interface exposing surface cropping and scaling capabilities is
    used to instantiate an interface extension for a
    :class:`~pywayland.protocol.wayland.WlSurface` object. This extended
    interface will then allow cropping and scaling the surface contents,
    effectively disconnecting the direct relationship between the buffer and
    the surface size.
    """
    name = "wp_viewporter"
    version = 1

    error = enum.Enum("error", {
        "viewport_exists": 0,
    })


@WpViewporter.request("", [])
def destroy(self):
    """Unbind from the cropping and scaling interface

    Informs the server that the client will not be using this protocol object
    anymore. This does not affect any other objects,
    :class:`~pywayland.protocol.viewporter.WpViewport` objects included.
    """
    self._marshal(0)
    self._destroy()


@WpViewporter.request("no", [WpViewport, WlSurface])
def get_viewport(self, surface):
    """Extend surface interface for crop and scale

    Instantiate an interface extension for the given
    :class:`~pywayland.protocol.wayland.WlSurface` to crop and scale its
    content. If the given :class:`~pywayland.protocol.wayland.WlSurface`
    already has a :class:`~pywayland.protocol.viewporter.WpViewport` object
    associated, the viewport_exists protocol error is raised.

    :param surface:
        the surface
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :returns:
        :class:`~pywayland.protocol.viewporter.WpViewport` -- the new viewport
        interface id
    """
    id = self._marshal_constructor(1, WpViewport, surface)
    return id


WpViewporter._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSeat

import enum


class ZxdgPopupV6(Interface):
    """Short-lived, popup surfaces for menus

    A popup surface is a short-lived, temporary surface. It can be used to
    implement for example menus, popovers, tooltips and other similar user
    interface concepts.

    A popup can be made to take an explicit grab. See :func:`XdgPopup.grab()
    <pywayland.protocol.xdg_shell.XdgPopup..grab>` for details.

    When the popup is dismissed, a popup_done event will be sent out, and at
    the same time the surface will be unmapped. See the
    :func:`XdgPopup.popup_done()
    <pywayland.protocol.xdg_shell.XdgPopup..popup_done>` event for details.

    Explicitly destroying the :class:`~pywayland.protocol.xdg_shell.XdgPopup`
    object will also dismiss the popup and unmap the surface. Clients that want
    to dismiss the popup when another surface of their own is clicked should
    dismiss the popup using the destroy request.

    The parent surface must have either the
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` or
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role.

    A newly created :class:`~pywayland.protocol.xdg_shell.XdgPopup` will be
    stacked on top of all previously created
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` surfaces associated with
    the same :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.

    The parent of an :class:`~pywayland.protocol.xdg_shell.XdgPopup` must be
    mapped (see the :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    description) before the :class:`~pywayland.protocol.xdg_shell.XdgPopup`
    itself.

    The x and y arguments passed when creating the popup object specify where
    the top left of the popup should be placed, relative to the local surface
    coordinates of the parent surface. See :func:`XdgSurface.get_popup()
    <pywayland.protocol.xdg_shell.XdgSurface..get_popup>`. An
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` must intersect with or be
    at least partially adjacent to its parent surface.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface..commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` state to take effect.
    """
    name = "zxdg_popup_v6"
    version = 1

    error = enum.Enum("error", {
        "invalid_grab": 0,
    })


@ZxdgPopupV6.request("", [])
def destroy(self):
    """Remove :class:`~pywayland.protocol.xdg_shell.XdgPopup` interface

    This destroys the popup. Explicitly destroying the
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` object will also dismiss
    the popup, and unmap the surface.

    If this :class:`~pywayland.protocol.xdg_shell.XdgPopup` is not the
    "topmost" popup, a protocol error will be sent.
    """
    self._marshal(0)
    self._destroy()


@ZxdgPopupV6.request("ou", [WlSeat, None])
def grab(self, seat, serial):
    """Make the popup take an explicit grab

    This request makes the created popup take an explicit grab. An explicit
    grab will be dismissed when the user dismisses the popup, or when the
    client destroys the :class:`~pywayland.protocol.xdg_shell.XdgPopup`. This
    can be done by the user clicking outside the surface, using the keyboard,
    or even locking the screen through closing the lid or a timeout.

    If the compositor denies the grab, the popup will be immediately dismissed.

    This request must be used in response to some sort of user action like a
    button press, key press, or touch down event. The serial number of the
    event should be passed as 'serial'.

    The parent of a grabbing popup must either be an
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface or another
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` with an explicit grab. If
    the parent is another :class:`~pywayland.protocol.xdg_shell.XdgPopup` it
    means that the popups are nested, with this popup now being the topmost
    popup.

    Nested popups must be destroyed in the reverse order they were created in,
    e.g. the only popup you are allowed to destroy at all times is the topmost
    one.

    When compositors choose to dismiss a popup, they may dismiss every nested
    grabbing popup as well. When a compositor dismisses popups, it will follow
    the same dismissing order as required from the client.

    The parent of a grabbing popup must either be another
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` with an active explicit
    grab, or an :class:`~pywayland.protocol.xdg_shell.XdgPopup` or
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`, if there are no
    explicit grabs already taken.

    If the topmost grabbing popup is destroyed, the grab will be returned to
    the parent of the popup, if that parent previously had an explicit grab.

    If the parent is a grabbing popup which has already been dismissed, this
    popup will be immediately dismissed. If the parent is a popup that did not
    take an explicit grab, an error will be raised.

    During a popup grab, the client owning the grab will receive pointer and
    touch events for all their surfaces as normal (similar to an "owner-events"
    grab in X11 parlance), while the top most grabbing popup will always have
    keyboard focus.

    :param seat:
        the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :param serial:
        the serial of the user event
    :type serial: `uint`
    """
    self._marshal(1, seat, serial)


@ZxdgPopupV6.event("iiii", [None, None, None, None])
def configure(self, x, y, width, height):
    """Configure the popup surface

    This event asks the popup surface to configure itself given the
    configuration. The configured state should not be applied immediately. See
    :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` for details.

    The x and y arguments represent the position the popup was placed at given
    the :class:`~pywayland.protocol.xdg_shell.XdgPositioner` rule, relative to
    the upper left corner of the window geometry of the parent surface.

    :param x:
        x position relative to parent surface window geometry
    :type x: `int`
    :param y:
        y position relative to parent surface window geometry
    :type y: `int`
    :param width:
        window geometry width
    :type width: `int`
    :param height:
        window geometry height
    :type height: `int`
    """
    self._post_event(0, x, y, width, height)


@ZxdgPopupV6.event("", [])
def popup_done(self):
    """Popup interaction is done

    The popup_done event is sent out when a popup is dismissed by the
    compositor. The client should destroy the
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` object at this point.
    """
    self._post_event(1)


ZxdgPopupV6._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2014, 2015 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlBuffer

import enum


class ZwpLinuxBufferParamsV1(Interface):
    """Parameters for creating a dmabuf-based :class:`~pywayland.protocol.wayland.WlBuffer`

    This temporary object is a collection of dmabufs and other parameters that
    together form a single logical buffer. The temporary object may eventually
    create one :class:`~pywayland.protocol.wayland.WlBuffer` unless cancelled
    by destroying it before requesting 'create'.

    Single-planar formats only require one dmabuf, however multi-planar formats
    may require more than one dmabuf. For all formats, an 'add' request must be
    called once per plane (even if the underlying dmabuf fd is identical).

    You must use consecutive plane indices ('plane_idx' argument for 'add')
    from zero to the number of planes used by the drm_fourcc format code. All
    planes required by the format must be given exactly once, but can be given
    in any order. Each plane index can be set only once.
    """
    name = "zwp_linux_buffer_params_v1"
    version = 3

    error = enum.Enum("error", {
        "already_used": 0,
        "plane_idx": 1,
        "plane_set": 2,
        "incomplete": 3,
        "invalid_format": 4,
        "invalid_dimensions": 5,
        "out_of_bounds": 6,
        "invalid_wl_buffer": 7,
    })

    flags = enum.Enum("flags", {
        "y_invert": 1,
        "interlaced": 2,
        "bottom_first": 4,
    })


@ZwpLinuxBufferParamsV1.request("", [])
def destroy(self):
    """Delete this object, used or not

    Cleans up the temporary data sent to the server for dmabuf-based
    :class:`~pywayland.protocol.wayland.WlBuffer` creation.
    """
    self._marshal(0)
    self._destroy()


@ZwpLinuxBufferParamsV1.request("huuuuu", [None, None, None, None, None, None])
def add(self, fd, plane_idx, offset, stride, modifier_hi, modifier_lo):
    """Add a dmabuf to the temporary set

    This request adds one dmabuf to the set in this
    :class:`ZwpLinuxBufferParamsV1`.

    The 64-bit unsigned value combined from modifier_hi and modifier_lo is the
    dmabuf layout modifier. DRM AddFB2 ioctl calls this the fb modifier, which
    is defined in drm_mode.h of Linux UAPI. This is an opaque token. Drivers
    use this token to express tiling, compression, etc. driver-specific
    modifications to the base format defined by the DRM fourcc code.

    Warning: It should be an error if the format/modifier pair was not
    advertised with the modifier event. This is not enforced yet because some
    implementations always accept DRM_FORMAT_MOD_INVALID. Also version 2 of
    this protocol does not have the modifier event.

    This request raises the PLANE_IDX error if plane_idx is too large. The
    error PLANE_SET is raised if attempting to set a plane that was already
    set.

    :param fd:
        dmabuf fd
    :type fd: `fd`
    :param plane_idx:
        plane index
    :type plane_idx: `uint`
    :param offset:
        offset in bytes
    :type offset: `uint`
    :param stride:
        stride in bytes
    :type stride: `uint`
    :param modifier_hi:
        high 32 bits of layout modifier
    :type modifier_hi: `uint`
    :param modifier_lo:
        low 32 bits of layout modifier
    :type modifier_lo: `uint`
    """
    self._marshal(1, fd, plane_idx, offset, stride, modifier_hi, modifier_lo)


@ZwpLinuxBufferParamsV1.request("iiuu", [None, None, None, None])
def create(self, width, height, format, flags):
    """Create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given dmabufs

    This asks for creation of a :class:`~pywayland.protocol.wayland.WlBuffer`
    from the added dmabuf buffers. The
    :class:`~pywayland.protocol.wayland.WlBuffer` is not created immediately
    but returned via the 'created' event if the dmabuf sharing succeeds. The
    sharing may fail at runtime for reasons a client cannot predict, in which
    case the 'failed' event is triggered.

    The 'format' argument is a DRM_FORMAT code, as defined by the libdrm's
    drm_fourcc.h. The Linux kernel's DRM sub-system is the authoritative source
    on how the format codes should work.

    The 'flags' is a bitfield of the flags defined in enum "flags". 'y_invert'
    means the that the image needs to be y-flipped.

    Flag 'interlaced' means that the frame in the buffer is not progressive as
    usual, but interlaced. An interlaced buffer as supported here must always
    contain both top and bottom fields. The top field always begins on the
    first pixel row. The temporal ordering between the two fields is top field
    first, unless 'bottom_first' is specified. It is undefined whether
    'bottom_first' is ignored if 'interlaced' is not set.

    This protocol does not convey any information about field rate, duration,
    or timing, other than the relative ordering between the two fields in one
    buffer. A compositor may have to estimate the intended field rate from the
    incoming buffer rate. It is undefined whether the time of receiving
    :func:`WlSurface.commit() <pywayland.protocol.wayland.WlSurface..commit>`
    with a new buffer attached, applying the
    :class:`~pywayland.protocol.wayland.WlSurface` state,
    :func:`WlSurface.frame() <pywayland.protocol.wayland.WlSurface..frame>`
    callback trigger, presentation, or any other point in the compositor cycle
    is used to measure the frame or field times. There is no support for
    detecting missed or late frames/fields/buffers either, and there is no
    support whatsoever for cooperating with interlaced compositor output.

    The composited image quality resulting from the use of interlaced buffers
    is explicitly undefined. A compositor may use elaborate hardware features
    or software to deinterlace and create progressive output frames from a
    sequence of interlaced input buffers, or it may produce substandard image
    quality. However, compositors that cannot guarantee reasonable image
    quality in all cases are recommended to just reject all interlaced buffers.

    Any argument errors, including non-positive width or height, mismatch
    between the number of planes and the format, bad format, bad offset or
    stride, may be indicated by fatal protocol errors: INCOMPLETE,
    INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS.

    Dmabuf import errors in the server that are not obvious client bugs are
    returned via the 'failed' event as non-fatal. This allows attempting dmabuf
    sharing and falling back in the client if it fails.

    This request can be sent only once in the object's lifetime, after which
    the only legal request is destroy. This object should be destroyed after
    issuing a 'create' request. Attempting to use this object after issuing
    'create' raises ALREADY_USED protocol error.

    It is not mandatory to issue 'create'. If a client wants to cancel the
    buffer creation, it can just destroy this object.

    :param width:
        base plane width in pixels
    :type width: `int`
    :param height:
        base plane height in pixels
    :type height: `int`
    :param format:
        DRM_FORMAT code
    :type format: `uint`
    :param flags:
        see enum flags
    :type flags: `uint`
    """
    self._marshal(2, width, height, format, flags)


@ZwpLinuxBufferParamsV1.request("2niiuu", [WlBuffer, None, None, None, None])
def create_immed(self, width, height, format, flags):
    """Immediately create a :class:`~pywayland.protocol.wayland.WlBuffer` from the given                      dmabufs

    This asks for immediate creation of a
    :class:`~pywayland.protocol.wayland.WlBuffer` by importing the added
    dmabufs.

    In case of import success, no event is sent from the server, and the
    :class:`~pywayland.protocol.wayland.WlBuffer` is ready to be used by the
    client.

    Upon import failure, either of the following may happen, as seen fit by the
    implementation: - the client is terminated with one of the following fatal
    protocol   errors:   - INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS,
    OUT_OF_BOUNDS,     in case of argument errors such as mismatch between the
    number     of planes and the format, bad format, non-positive width or
    height, or bad offset or stride.   - INVALID_WL_BUFFER, in case the cause
    for failure is unknown or     plaform specific. - the server creates an
    invalid :class:`~pywayland.protocol.wayland.WlBuffer`, marks it as failed
    and   sends a 'failed' event to the client. The result of using this
    invalid :class:`~pywayland.protocol.wayland.WlBuffer` as an argument in any
    request by the client is   defined by the compositor implementation.

    This takes the same arguments as a 'create' request, and obeys the same
    restrictions.

    :param width:
        base plane width in pixels
    :type width: `int`
    :param height:
        base plane height in pixels
    :type height: `int`
    :param format:
        DRM_FORMAT code
    :type format: `uint`
    :param flags:
        see enum flags
    :type flags: `uint`
    :returns:
        :class:`~pywayland.protocol.wayland.WlBuffer` -- id for the newly
        created :class:`~pywayland.protocol.wayland.WlBuffer`
    """
    buffer_id = self._marshal_constructor(3, WlBuffer, width, height, format, flags)
    return buffer_id


@ZwpLinuxBufferParamsV1.event("n", [WlBuffer])
def created(self, buffer):
    """Buffer creation succeeded

    This event indicates that the attempted buffer creation was successful. It
    provides the new :class:`~pywayland.protocol.wayland.WlBuffer` referencing
    the dmabuf(s).

    Upon receiving this event, the client should destroy the
    zlinux_dmabuf_params object.

    :param buffer:
        the newly created :class:`~pywayland.protocol.wayland.WlBuffer`
    :type buffer: :class:`~pywayland.protocol.wayland.WlBuffer`
    """
    self._post_event(0, buffer)


@ZwpLinuxBufferParamsV1.event("", [])
def failed(self):
    """Buffer creation failed

    This event indicates that the attempted buffer creation has failed. It
    usually means that one of the dmabuf constraints has not been fulfilled.

    Upon receiving this event, the client should destroy the
    zlinux_buffer_params object.
    """
    self._post_event(1)


ZwpLinuxBufferParamsV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_shell_surface import WlShellSurface
from .wl_surface import WlSurface

import enum


class WlShell(Interface):
    """Create desktop-style surfaces

    This interface is implemented by servers that provide desktop-style user
    interfaces.

    It allows clients to associate a
    :class:`~pywayland.protocol.wayland.WlShellSurface` with a basic surface.

    Note! This protocol is deprecated and not intended for production use. For
    desktop-style user interfaces, use xdg_shell.
    """
    name = "wl_shell"
    version = 1

    error = enum.Enum("error", {
        "role": 0,
    })


@WlShell.request("no", [WlShellSurface, WlSurface])
def get_shell_surface(self, surface):
    """Create a shell surface from a surface

    Create a shell surface for an existing surface. This gives the
    :class:`~pywayland.protocol.wayland.WlSurface` the role of a shell surface.
    If the :class:`~pywayland.protocol.wayland.WlSurface` already has another
    role, it raises a protocol error.

    Only one shell surface can be associated with a given surface.

    :param surface:
        surface to be given the shell surface role
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :returns:
        :class:`~pywayland.protocol.wayland.WlShellSurface` -- shell surface to
        create
    """
    id = self._marshal_constructor(0, WlShellSurface, surface)
    return id


WlShell._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface

import enum


class XdgPositioner(Interface):
    """Child surface positioner

    The :class:`XdgPositioner` provides a collection of rules for the placement
    of a child surface relative to a parent surface. Rules can be defined to
    ensure the child surface remains within the visible area's borders, and to
    specify how the child surface changes its position, such as sliding along
    an axis, or flipping around a rectangle. These positioner-created rules are
    constrained by the requirement that a child surface must intersect with or
    be at least partially adjacent to its parent surface.

    See the various requests for details about possible rules.

    At the time of the request, the compositor makes a copy of the rules
    specified by the :class:`XdgPositioner`. Thus, after the request is
    complete the :class:`XdgPositioner` object can be destroyed or reused;
    further changes to the object will have no effect on previous usages.

    For an :class:`XdgPositioner` object to be considered complete, it must
    have a non-zero size set by set_size, and a non-zero anchor rectangle set
    by set_anchor_rect. Passing an incomplete :class:`XdgPositioner` object
    when positioning a surface raises an error.
    """
    name = "xdg_positioner"
    version = 2

    error = enum.Enum("error", {
        "invalid_input": 0,
    })

    anchor = enum.Enum("anchor", {
        "none": 0,
        "top": 1,
        "bottom": 2,
        "left": 3,
        "right": 4,
        "top_left": 5,
        "bottom_left": 6,
        "top_right": 7,
        "bottom_right": 8,
    })

    gravity = enum.Enum("gravity", {
        "none": 0,
        "top": 1,
        "bottom": 2,
        "left": 3,
        "right": 4,
        "top_left": 5,
        "bottom_left": 6,
        "top_right": 7,
        "bottom_right": 8,
    })

    constraint_adjustment = enum.Enum("constraint_adjustment", {
        "none": 0,
        "slide_x": 1,
        "slide_y": 2,
        "flip_x": 4,
        "flip_y": 8,
        "resize_x": 16,
        "resize_y": 32,
    })


@XdgPositioner.request("", [])
def destroy(self):
    """Destroy the :class:`XdgPositioner` object

    Notify the compositor that the :class:`XdgPositioner` will no longer be
    used.
    """
    self._marshal(0)
    self._destroy()


@XdgPositioner.request("ii", [None, None])
def set_size(self, width, height):
    """Set the size of the to-be positioned rectangle

    Set the size of the surface that is to be positioned with the positioner
    object. The size is in surface-local coordinates and corresponds to the
    window geometry. See :func:`XdgSurface.set_window_geometry()
    <pywayland.protocol.xdg_shell.XdgSurface..set_window_geometry>`.

    If a zero or negative size is set the invalid_input error is raised.

    :param width:
        width of positioned rectangle
    :type width: `int`
    :param height:
        height of positioned rectangle
    :type height: `int`
    """
    self._marshal(1, width, height)


@XdgPositioner.request("iiii", [None, None, None, None])
def set_anchor_rect(self, x, y, width, height):
    """Set the anchor rectangle within the parent surface

    Specify the anchor rectangle within the parent surface that the child
    surface will be placed relative to. The rectangle is relative to the window
    geometry as defined by :func:`XdgSurface.set_window_geometry()
    <pywayland.protocol.xdg_shell.XdgSurface..set_window_geometry>` of the
    parent surface.

    When the :class:`XdgPositioner` object is used to position a child surface,
    the anchor rectangle may not extend outside the window geometry of the
    positioned child's parent surface.

    If a negative size is set the invalid_input error is raised.

    :param x:
        x position of anchor rectangle
    :type x: `int`
    :param y:
        y position of anchor rectangle
    :type y: `int`
    :param width:
        width of anchor rectangle
    :type width: `int`
    :param height:
        height of anchor rectangle
    :type height: `int`
    """
    self._marshal(2, x, y, width, height)


@XdgPositioner.request("u", [None])
def set_anchor(self, anchor):
    """Set anchor rectangle anchor

    Defines the anchor point for the anchor rectangle. The specified anchor is
    used derive an anchor point that the child surface will be positioned
    relative to. If a corner anchor is set (e.g. 'top_left' or 'bottom_right'),
    the anchor point will be at the specified corner; otherwise, the derived
    anchor point will be centered on the specified edge, or in the center of
    the anchor rectangle if no edge is specified.

    :param anchor:
        anchor
    :type anchor: `uint`
    """
    self._marshal(3, anchor)


@XdgPositioner.request("u", [None])
def set_gravity(self, gravity):
    """Set child surface gravity

    Defines in what direction a surface should be positioned, relative to the
    anchor point of the parent surface. If a corner gravity is specified (e.g.
    'bottom_right' or 'top_left'), then the child surface will be placed
    towards the specified gravity; otherwise, the child surface will be
    centered over the anchor point on any axis that had no gravity specified.

    :param gravity:
        gravity direction
    :type gravity: `uint`
    """
    self._marshal(4, gravity)


@XdgPositioner.request("u", [None])
def set_constraint_adjustment(self, constraint_adjustment):
    """Set the adjustment to be done when constrained

    Specify how the window should be positioned if the originally intended
    position caused the surface to be constrained, meaning at least partially
    outside positioning boundaries set by the compositor. The adjustment is set
    by constructing a bitmask describing the adjustment to be made when the
    surface is constrained on that axis.

    If no bit for one axis is set, the compositor will assume that the child
    surface should not change its position on that axis when constrained.

    If more than one bit for one axis is set, the order of how adjustments are
    applied is specified in the corresponding adjustment descriptions.

    The default adjustment is none.

    :param constraint_adjustment:
        bit mask of constraint adjustments
    :type constraint_adjustment: `uint`
    """
    self._marshal(5, constraint_adjustment)


@XdgPositioner.request("ii", [None, None])
def set_offset(self, x, y):
    """Set surface position offset

    Specify the surface position offset relative to the position of the anchor
    on the anchor rectangle and the anchor on the surface. For example if the
    anchor of the anchor rectangle is at (x, y), the surface has the gravity
    bottom|right, and the offset is (ox, oy), the calculated surface position
    will be (x + ox, y + oy). The offset position of the surface is the one
    used for constraint testing. See set_constraint_adjustment.

    An example use case is placing a popup menu on top of a user interface
    element, while aligning the user interface element of the parent surface
    with some user interface element placed somewhere in the popup surface.

    :param x:
        surface position x offset
    :type x: `int`
    :param y:
        surface position y offset
    :type y: `int`
    """
    self._marshal(6, x, y)


XdgPositioner._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from .zwp_primary_selection_offer_v1 import ZwpPrimarySelectionOfferV1
from .zwp_primary_selection_source_v1 import ZwpPrimarySelectionSourceV1


class ZwpPrimarySelectionDeviceV1(Interface):
    name = "zwp_primary_selection_device_v1"
    version = 1


@ZwpPrimarySelectionDeviceV1.request("?ou", [ZwpPrimarySelectionSourceV1, None])
def set_selection(self, source, serial):
    """Set the primary selection

    Replaces the current selection. The previous owner of the primary selection
    will receive a wp_primary_selection_source.cancelled event.

    To unset the selection, set the source to NULL.

    :param source:
    :type source: :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1` or `None`
    :param serial:
        serial of the event that triggered this request
    :type serial: `uint`
    """
    self._marshal(0, source, serial)


@ZwpPrimarySelectionDeviceV1.request("", [])
def destroy(self):
    """Destroy the primary selection device

    Destroy the primary selection device.
    """
    self._marshal(1)
    self._destroy()


@ZwpPrimarySelectionDeviceV1.event("n", [ZwpPrimarySelectionOfferV1])
def data_offer(self, offer):
    """Introduce a new wp_primary_selection_offer

    Introduces a new wp_primary_selection_offer object that may be used to
    receive the current primary selection. Immediately following this event,
    the new wp_primary_selection_offer object will send
    wp_primary_selection_offer.offer events to describe the offered mime types.

    :param offer:
    :type offer: :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1`
    """
    self._post_event(0, offer)


@ZwpPrimarySelectionDeviceV1.event("?o", [ZwpPrimarySelectionOfferV1])
def selection(self, id):
    """Advertise a new primary selection

    The wp_primary_selection_device.selection event is sent to notify the
    client of a new primary selection. This event is sent after the
    wp_primary_selection.data_offer event introducing this object, and after
    the offer has announced its mimetypes through
    wp_primary_selection_offer.offer.

    The data_offer is valid until a new offer or NULL is received or until the
    client loses keyboard focus. The client must destroy the previous selection
    data_offer, if any, upon receiving this event.

    :param id:
    :type id: :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1` or `None`
    """
    self._post_event(1, id)


ZwpPrimarySelectionDeviceV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface


class WlDestructor(Interface):
    """Destructor object

    An interface object with a destructor request.

    And a multiline description.
    """
    name = "wl_destructor"
    version = 1


@WlDestructor.request("niiiiu", [WlDestructor, None, None, None, None, None])
def create_interface(self, x, y, width, height, format):
    """Create another interface

    Create a :class:`WlDestructor` interface object

    :param x:
    :type x: `int`
    :param y:
    :type y: `int`
    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    :param format:
    :type format: `uint`
    :returns:
        :class:`WlDestructor`
    """
    id = self._marshal_constructor(0, WlDestructor, x, y, width, height, format)
    return id


@WlDestructor.request("", [])
def destroy(self):
    """Destroy the interface

    Destroy the created interface.
    """
    self._marshal(1)
    self._destroy()


WlDestructor._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface


class ZwpPrimarySelectionOfferV1(Interface):
    """Offer to transfer primary selection contents

    A wp_primary_selection_offer represents an offer to transfer the contents
    of the primary selection clipboard to the client. Similar to
    :class:`~pywayland.protocol.wayland.WlDataOffer`, the offer also describes
    the mime types that the data can be converted to and provides the
    mechanisms for transferring the data directly to the client.
    """
    name = "zwp_primary_selection_offer_v1"
    version = 1


@ZwpPrimarySelectionOfferV1.request("sh", [None, None])
def receive(self, mime_type, fd):
    """Request that the data is transferred

    To transfer the contents of the primary selection clipboard, the client
    issues this request and indicates the mime type that it wants to receive.
    The transfer happens through the passed file descriptor (typically created
    with the pipe system call). The source client writes the data in the mime
    type representation requested and then closes the file descriptor.

    The receiving client reads from the read end of the pipe until EOF and
    closes its end, at which point the transfer is complete.

    :param mime_type:
    :type mime_type: `string`
    :param fd:
    :type fd: `fd`
    """
    self._marshal(0, mime_type, fd)


@ZwpPrimarySelectionOfferV1.request("", [])
def destroy(self):
    """Destroy the primary selection offer

    Destroy the primary selection offer.
    """
    self._marshal(1)
    self._destroy()


@ZwpPrimarySelectionOfferV1.event("s", [None])
def offer(self, mime_type):
    """Advertise offered mime type

    Sent immediately after creating announcing the wp_primary_selection_offer
    through wp_primary_selection_device.data_offer. One event is sent per
    offered mime type.

    :param mime_type:
    :type mime_type: `string`
    """
    self._post_event(0, mime_type)


ZwpPrimarySelectionOfferV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlOutput
from .zxdg_output_v1 import ZxdgOutputV1


class ZxdgOutputManagerV1(Interface):
    """Manage xdg_output objects

    A global factory interface for xdg_output objects.
    """
    name = "zxdg_output_manager_v1"
    version = 3


@ZxdgOutputManagerV1.request("", [])
def destroy(self):
    """Destroy the xdg_output_manager object

    Using this request a client can tell the server that it is not going to use
    the xdg_output_manager object anymore.

    Any objects already created through this instance are not affected.
    """
    self._marshal(0)
    self._destroy()


@ZxdgOutputManagerV1.request("no", [ZxdgOutputV1, WlOutput])
def get_xdg_output(self, output):
    """Create an xdg output from a :class:`~pywayland.protocol.wayland.WlOutput`

    This creates a new xdg_output object for the given
    :class:`~pywayland.protocol.wayland.WlOutput`.

    :param output:
    :type output: :class:`~pywayland.protocol.wayland.WlOutput`
    :returns:
        :class:`~pywayland.protocol.xdg_output_unstable_v1.ZxdgOutputV1`
    """
    id = self._marshal_constructor(1, ZxdgOutputV1, output)
    return id


ZxdgOutputManagerV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_data_offer import WlDataOffer
from .wl_data_source import WlDataSource
from .wl_surface import WlSurface

import enum


class WlDataDevice(Interface):
    """Data transfer device

    There is one :class:`WlDataDevice` per seat which can be obtained from the
    global :class:`~pywayland.protocol.wayland.WlDataDeviceManager` singleton.

    A :class:`WlDataDevice` provides access to inter-client data transfer
    mechanisms such as copy-and-paste and drag-and-drop.
    """
    name = "wl_data_device"
    version = 3

    error = enum.Enum("error", {
        "role": 0,
    })


@WlDataDevice.request("?oo?ou", [WlDataSource, WlSurface, WlSurface, None])
def start_drag(self, source, origin, icon, serial):
    """Start drag-and-drop operation

    This request asks the compositor to start a drag-and-drop operation on
    behalf of the client.

    The source argument is the data source that provides the data for the
    eventual data transfer. If source is NULL, enter, leave and motion events
    are sent only to the client that initiated the drag and the client is
    expected to handle the data passing internally.

    The origin surface is the surface where the drag originates and the client
    must have an active implicit grab that matches the serial.

    The icon surface is an optional (can be NULL) surface that provides an icon
    to be moved around with the cursor.  Initially, the top-left corner of the
    icon surface is placed at the cursor hotspot, but subsequent
    :func:`WlSurface.attach() <pywayland.protocol.wayland.WlSurface..attach>`
    request can move the relative position. Attach requests must be confirmed
    with :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface..commit>` as usual. The icon surface
    is given the role of a drag-and-drop icon. If the icon surface already has
    another role, it raises a protocol error.

    The current and pending input regions of the icon
    :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
    :func:`WlSurface.set_input_region()
    <pywayland.protocol.wayland.WlSurface..set_input_region>` is ignored until
    the :class:`~pywayland.protocol.wayland.WlSurface` is no longer used as the
    icon surface. When the use as an icon ends, the current and pending input
    regions become undefined, and the
    :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

    :param source:
        data source for the eventual transfer
    :type source: :class:`~pywayland.protocol.wayland.WlDataSource` or `None`
    :param origin:
        surface where the drag originates
    :type origin: :class:`~pywayland.protocol.wayland.WlSurface`
    :param icon:
        drag-and-drop icon surface
    :type icon: :class:`~pywayland.protocol.wayland.WlSurface` or `None`
    :param serial:
        serial number of the implicit grab on the origin
    :type serial: `uint`
    """
    self._marshal(0, source, origin, icon, serial)


@WlDataDevice.request("?ou", [WlDataSource, None])
def set_selection(self, source, serial):
    """Copy data to the selection

    This request asks the compositor to set the selection to the data from the
    source on behalf of the client.

    To unset the selection, set the source to NULL.

    :param source:
        data source for the selection
    :type source: :class:`~pywayland.protocol.wayland.WlDataSource` or `None`
    :param serial:
        serial number of the event that triggered this request
    :type serial: `uint`
    """
    self._marshal(1, source, serial)


@WlDataDevice.request("2", [])
def release(self):
    """Destroy data device

    This request destroys the data device.
    """
    self._marshal(2)
    self._destroy()


@WlDataDevice.event("n", [WlDataOffer])
def data_offer(self, id):
    """Introduce a new :class:`~pywayland.protocol.wayland.WlDataOffer`

    The data_offer event introduces a new
    :class:`~pywayland.protocol.wayland.WlDataOffer` object, which will
    subsequently be used in either the data_device.enter event (for drag-and-
    drop) or the data_device.selection event (for selections).  Immediately
    following the data_device_data_offer event, the new data_offer object will
    send out data_offer.offer events to describe the mime types it offers.

    :param id:
        the new data_offer object
    :type id: :class:`~pywayland.protocol.wayland.WlDataOffer`
    """
    self._post_event(0, id)


@WlDataDevice.event("uoff?o", [None, WlSurface, None, None, WlDataOffer])
def enter(self, serial, surface, x, y, id):
    """Initiate drag-and-drop session

    This event is sent when an active drag-and-drop pointer enters a surface
    owned by the client.  The position of the pointer at enter time is provided
    by the x and y arguments, in surface-local coordinates.

    :param serial:
        serial number of the enter event
    :type serial: `uint`
    :param surface:
        client surface entered
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param x:
        surface-local x coordinate
    :type x: `fixed`
    :param y:
        surface-local y coordinate
    :type y: `fixed`
    :param id:
        source data_offer object
    :type id: :class:`~pywayland.protocol.wayland.WlDataOffer` or `None`
    """
    self._post_event(1, serial, surface, x, y, id)


@WlDataDevice.event("", [])
def leave(self):
    """End drag-and-drop session

    This event is sent when the drag-and-drop pointer leaves the surface and
    the session ends.  The client must destroy the
    :class:`~pywayland.protocol.wayland.WlDataOffer` introduced at enter time
    at this point.
    """
    self._post_event(2)


@WlDataDevice.event("uff", [None, None, None])
def motion(self, time, x, y):
    """Drag-and-drop session motion

    This event is sent when the drag-and-drop pointer moves within the
    currently focused surface. The new position of the pointer is provided by
    the x and y arguments, in surface-local coordinates.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param x:
        surface-local x coordinate
    :type x: `fixed`
    :param y:
        surface-local y coordinate
    :type y: `fixed`
    """
    self._post_event(3, time, x, y)


@WlDataDevice.event("", [])
def drop(self):
    """End drag-and-drop session successfully

    The event is sent when a drag-and-drop operation is ended because the
    implicit grab is removed.

    The drag-and-drop destination is expected to honor the last action received
    through :func:`WlDataOffer.action()
    <pywayland.protocol.wayland.WlDataOffer..action>`, if the resulting action
    is "copy" or "move", the destination can still perform
    :func:`WlDataOffer.receive()
    <pywayland.protocol.wayland.WlDataOffer..receive>` requests, and is
    expected to end all transfers with a :func:`WlDataOffer.finish()
    <pywayland.protocol.wayland.WlDataOffer..finish>` request.

    If the resulting action is "ask", the action will not be considered final.
    The drag-and-drop destination is expected to perform one last
    :func:`WlDataOffer.set_actions()
    <pywayland.protocol.wayland.WlDataOffer..set_actions>` request, or
    :func:`WlDataOffer.destroy()
    <pywayland.protocol.wayland.WlDataOffer..destroy>` in order to cancel the
    operation.
    """
    self._post_event(4)


@WlDataDevice.event("?o", [WlDataOffer])
def selection(self, id):
    """Advertise new selection

    The selection event is sent out to notify the client of a new
    :class:`~pywayland.protocol.wayland.WlDataOffer` for the selection for this
    device.  The data_device.data_offer and the data_offer.offer events are
    sent out immediately before this event to introduce the data offer object.
    The selection event is sent to a client immediately before receiving
    keyboard focus and when a new selection is set while the client has
    keyboard focus.  The data_offer is valid until a new data_offer or NULL is
    received or until the client loses keyboard focus.  The client must destroy
    the previous selection data_offer, if any, upon receiving this event.

    :param id:
        selection data_offer object
    :type id: :class:`~pywayland.protocol.wayland.WlDataOffer` or `None`
    """
    self._post_event(5, id)


WlDataDevice._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_keyboard import WlKeyboard
from .wl_pointer import WlPointer
from .wl_touch import WlTouch

import enum


class WlSeat(Interface):
    """Group of input devices

    A seat is a group of keyboards, pointer and touch devices. This object is
    published as a global during start up, or when such a device is hot
    plugged.  A seat typically has a pointer and maintains a keyboard focus and
    a pointer focus.
    """
    name = "wl_seat"
    version = 7

    capability = enum.Enum("capability", {
        "pointer": 1,
        "keyboard": 2,
        "touch": 4,
    })


@WlSeat.request("n", [WlPointer])
def get_pointer(self):
    """Return pointer object

    The ID provided will be initialized to the
    :class:`~pywayland.protocol.wayland.WlPointer` interface for this seat.

    This request only takes effect if the seat has the pointer capability, or
    has had the pointer capability in the past. It is a protocol violation to
    issue this request on a seat that has never had the pointer capability.

    :returns:
        :class:`~pywayland.protocol.wayland.WlPointer` -- seat pointer
    """
    id = self._marshal_constructor(0, WlPointer)
    return id


@WlSeat.request("n", [WlKeyboard])
def get_keyboard(self):
    """Return keyboard object

    The ID provided will be initialized to the
    :class:`~pywayland.protocol.wayland.WlKeyboard` interface for this seat.

    This request only takes effect if the seat has the keyboard capability, or
    has had the keyboard capability in the past. It is a protocol violation to
    issue this request on a seat that has never had the keyboard capability.

    :returns:
        :class:`~pywayland.protocol.wayland.WlKeyboard` -- seat keyboard
    """
    id = self._marshal_constructor(1, WlKeyboard)
    return id


@WlSeat.request("n", [WlTouch])
def get_touch(self):
    """Return touch object

    The ID provided will be initialized to the
    :class:`~pywayland.protocol.wayland.WlTouch` interface for this seat.

    This request only takes effect if the seat has the touch capability, or has
    had the touch capability in the past. It is a protocol violation to issue
    this request on a seat that has never had the touch capability.

    :returns:
        :class:`~pywayland.protocol.wayland.WlTouch` -- seat touch interface
    """
    id = self._marshal_constructor(2, WlTouch)
    return id


@WlSeat.request("5", [])
def release(self):
    """Release the seat object

    Using this request a client can tell the server that it is not going to use
    the seat object anymore.
    """
    self._marshal(3)
    self._destroy()


@WlSeat.event("u", [None])
def capabilities(self, capabilities):
    """Seat capabilities changed

    This is emitted whenever a seat gains or loses the pointer, keyboard or
    touch capabilities.  The argument is a capability enum containing the
    complete set of capabilities this seat has.

    When the pointer capability is added, a client may create a
    :class:`~pywayland.protocol.wayland.WlPointer` object using the
    :func:`WlSeat.get_pointer()` request. This object will receive pointer
    events until the capability is removed in the future.

    When the pointer capability is removed, a client should destroy the
    :class:`~pywayland.protocol.wayland.WlPointer` objects associated with the
    seat where the capability was removed, using the :func:`WlPointer.release()
    <pywayland.protocol.wayland.WlPointer..release>` request. No further
    pointer events will be received on these objects.

    In some compositors, if a seat regains the pointer capability and a client
    has a previously obtained :class:`~pywayland.protocol.wayland.WlPointer`
    object of version 4 or less, that object may start sending pointer events
    again. This behavior is considered a misinterpretation of the intended
    behavior and must not be relied upon by the client.
    :class:`~pywayland.protocol.wayland.WlPointer` objects of version 5 or
    later must not send events if created before the most recent event
    notifying the client of an added pointer capability.

    The above behavior also applies to
    :class:`~pywayland.protocol.wayland.WlKeyboard` and
    :class:`~pywayland.protocol.wayland.WlTouch` with the keyboard and touch
    capabilities, respectively.

    :param capabilities:
        capabilities of the seat
    :type capabilities: `uint`
    """
    self._post_event(0, capabilities)


@WlSeat.event("2s", [None])
def name(self, name):
    """Unique identifier for this seat

    In a multiseat configuration this can be used by the client to help
    identify which physical devices the seat represents. Based on the seat
    configuration used by the compositor.

    :param name:
        seat identifier
    :type name: `string`
    """
    self._post_event(1, name)


WlSeat._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface


class WlRegion(Interface):
    """Region interface

    A region object describes an area.

    Region objects are used to describe the opaque and input regions of a
    surface.
    """
    name = "wl_region"
    version = 1


@WlRegion.request("", [])
def destroy(self):
    """Destroy region

    Destroy the region.  This will invalidate the object ID.
    """
    self._marshal(0)
    self._destroy()


@WlRegion.request("iiii", [None, None, None, None])
def add(self, x, y, width, height):
    """Add rectangle to region

    Add the specified rectangle to the region.

    :param x:
        region-local x coordinate
    :type x: `int`
    :param y:
        region-local y coordinate
    :type y: `int`
    :param width:
        rectangle width
    :type width: `int`
    :param height:
        rectangle height
    :type height: `int`
    """
    self._marshal(1, x, y, width, height)


@WlRegion.request("iiii", [None, None, None, None])
def subtract(self, x, y, width, height):
    """Subtract rectangle from region

    Subtract the specified rectangle from the region.

    :param x:
        region-local x coordinate
    :type x: `int`
    :param y:
        region-local y coordinate
    :type y: `int`
    :param width:
        rectangle width
    :type width: `int`
    :param height:
        rectangle height
    :type height: `int`
    """
    self._marshal(2, x, y, width, height)


WlRegion._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface
from ..wayland import WlSurface


class ZwpPointerGestureSwipeV1(Interface):
    """A swipe gesture object

    A swipe gesture object notifies a client about a multi-finger swipe gesture
    detected on an indirect input device such as a touchpad. The gesture is
    usually initiated by multiple fingers moving in the same direction but once
    initiated the direction may change. The precise conditions of when such a
    gesture is detected are implementation-dependent.

    A gesture consists of three stages: begin, update (optional) and end. There
    cannot be multiple simultaneous pinch or swipe gestures on a same
    pointer/seat, how compositors prevent these situations is implementation-
    dependent.

    A gesture may be cancelled by the compositor or the hardware. Clients
    should not consider performing permanent or irreversible actions until the
    end of a gesture has been received.
    """
    name = "zwp_pointer_gesture_swipe_v1"
    version = 2


@ZwpPointerGestureSwipeV1.request("", [])
def destroy(self):
    """Destroy the pointer swipe gesture object
    """
    self._marshal(0)
    self._destroy()


@ZwpPointerGestureSwipeV1.event("uuou", [None, None, WlSurface, None])
def begin(self, serial, time, surface, fingers):
    """Multi-finger swipe begin

    This event is sent when a multi-finger swipe gesture is detected on the
    device.

    :param serial:
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param surface:
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    :param fingers:
        number of fingers
    :type fingers: `uint`
    """
    self._post_event(0, serial, time, surface, fingers)


@ZwpPointerGestureSwipeV1.event("uff", [None, None, None])
def update(self, time, dx, dy):
    """Multi-finger swipe motion

    This event is sent when a multi-finger swipe gesture changes the position
    of the logical center.

    The dx and dy coordinates are relative coordinates of the logical center of
    the gesture compared to the previous event.

    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param dx:
        delta x coordinate in surface coordinate space
    :type dx: `fixed`
    :param dy:
        delta y coordinate in surface coordinate space
    :type dy: `fixed`
    """
    self._post_event(1, time, dx, dy)


@ZwpPointerGestureSwipeV1.event("uui", [None, None, None])
def end(self, serial, time, cancelled):
    """Multi-finger swipe end

    This event is sent when a multi-finger swipe gesture ceases to be valid.
    This may happen when one or more fingers are lifted or the gesture is
    cancelled.

    When a gesture is cancelled, the client should undo state changes caused by
    this gesture. What causes a gesture to be cancelled is implementation-
    dependent.

    :param serial:
    :type serial: `uint`
    :param time:
        timestamp with millisecond granularity
    :type time: `uint`
    :param cancelled:
        1 if the gesture was cancelled, 0 otherwise
    :type cancelled: `int`
    """
    self._post_event(2, serial, time, cancelled)


ZwpPointerGestureSwipeV1._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.interface import Interface
from .zxdg_popup_v6 import ZxdgPopupV6
from .zxdg_positioner_v6 import ZxdgPositionerV6
from .zxdg_toplevel_v6 import ZxdgToplevelV6

import enum


class ZxdgSurfaceV6(Interface):
    """Desktop user interface surface base interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides a base set of functionality required to construct user
    interface elements requiring management by the compositor, such as toplevel
    windows, menus, etc. The types of functionality are split into
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` roles.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` does not set
    the role for a :class:`~pywayland.protocol.wayland.WlSurface`. In order to
    map an :class:`~pywayland.protocol.xdg_shell.XdgSurface`, the client must
    create a role-specific object using, e.g., get_toplevel, get_popup. The
    :class:`~pywayland.protocol.wayland.WlSurface` for any given
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` can have at most one
    role, and may not be assigned any role not based on
    :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

    A role must be assigned before any other requests are made to the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` object.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface..commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state to take effect.

    Creating an :class:`~pywayland.protocol.xdg_shell.XdgSurface` from a
    :class:`~pywayland.protocol.wayland.WlSurface` which has a buffer attached
    or committed is a client error, and any attempts by a client to attach or
    manipulate a buffer prior to the first :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` call must also be
    treated as errors.

    For a surface to be mapped by the compositor, the following conditions must
    be met: (1) the client has assigned a
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` based role to the
    surface, (2) the client has set and committed the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` state and the role
    dependent state to the surface and (3) the client has committed a buffer to
    the surface.
    """
    name = "zxdg_surface_v6"
    version = 1

    error = enum.Enum("error", {
        "not_constructed": 1,
        "already_constructed": 2,
        "unconfigured_buffer": 3,
    })


@ZxdgSurfaceV6.request("", [])
def destroy(self):
    """Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface`

    Destroy the :class:`~pywayland.protocol.xdg_shell.XdgSurface` object. An
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` must only be destroyed
    after its role object has been destroyed.
    """
    self._marshal(0)
    self._destroy()


@ZxdgSurfaceV6.request("n", [ZxdgToplevelV6])
def get_toplevel(self):
    """Assign the :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surface role

    This creates an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` object
    for the given :class:`~pywayland.protocol.xdg_shell.XdgSurface` and gives
    the associated :class:`~pywayland.protocol.wayland.WlSurface` the
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role.

    See the documentation of :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
    for more details about what an
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is and how it is used.

    :returns:
        :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgToplevelV6`
    """
    id = self._marshal_constructor(1, ZxdgToplevelV6)
    return id


@ZxdgSurfaceV6.request("noo", [ZxdgPopupV6, ZxdgSurfaceV6, ZxdgPositionerV6])
def get_popup(self, parent, positioner):
    """Assign the :class:`~pywayland.protocol.xdg_shell.XdgPopup` surface role

    This creates an :class:`~pywayland.protocol.xdg_shell.XdgPopup` object for
    the given :class:`~pywayland.protocol.xdg_shell.XdgSurface` and gives the
    associated :class:`~pywayland.protocol.wayland.WlSurface` the
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` role.

    See the documentation of :class:`~pywayland.protocol.xdg_shell.XdgPopup`
    for more details about what an
    :class:`~pywayland.protocol.xdg_shell.XdgPopup` is and how it is used.

    :param parent:
    :type parent: :class:`ZxdgSurfaceV6`
    :param positioner:
    :type positioner: :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPositionerV6`
    :returns:
        :class:`~pywayland.protocol.xdg_shell_unstable_v6.ZxdgPopupV6`
    """
    id = self._marshal_constructor(2, ZxdgPopupV6, parent, positioner)
    return id


@ZxdgSurfaceV6.request("iiii", [None, None, None, None])
def set_window_geometry(self, x, y, width, height):
    """Set the new window geometry

    The window geometry of a surface is its "visible bounds" from the user's
    perspective. Client-side decorations often have invisible portions like
    drop-shadows which should be ignored for the purposes of aligning, placing
    and constraining windows.

    The window geometry is double buffered, and will be applied at the time
    :func:`WlSurface.commit() <pywayland.protocol.wayland.WlSurface..commit>`
    of the corresponding :class:`~pywayland.protocol.wayland.WlSurface` is
    called.

    Once the window geometry of the surface is set, it is not possible to unset
    it, and it will remain the same until set_window_geometry is called again,
    even if a new subsurface or buffer is attached.

    If never set, the value is the full bounds of the surface, including any
    subsurfaces. This updates dynamically on every commit. This unset is meant
    for extremely simple clients.

    The arguments are given in the surface-local coordinate space of the
    :class:`~pywayland.protocol.wayland.WlSurface` associated with this
    :class:`~pywayland.protocol.xdg_shell.XdgSurface`.

    The width and height must be greater than zero. Setting an invalid size
    will raise an error. When applied, the effective window geometry will be
    the set window geometry clamped to the bounding rectangle of the combined
    geometry of the surface of the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` and the associated
    subsurfaces.

    :param x:
    :type x: `int`
    :param y:
    :type y: `int`
    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    """
    self._marshal(3, x, y, width, height)


@ZxdgSurfaceV6.request("u", [None])
def ack_configure(self, serial):
    """Ack a configure event

    When a configure event is received, if a client commits the surface in
    response to the configure event, then the client must make an ack_configure
    request sometime before the commit request, passing along the serial of the
    configure event.

    For instance, for toplevel surfaces the compositor might use this
    information to move a surface to the top left only when the client has
    drawn itself for the maximized or fullscreen state.

    If the client receives multiple configure events before it can respond to
    one, it only has to ack the last configure event.

    A client is not required to commit immediately after sending an
    ack_configure request - it may even ack_configure several times before its
    next surface commit.

    A client may send multiple ack_configure requests before committing, but
    only the last request sent before a commit indicates which configure event
    the client really is responding to.

    :param serial:
        the serial from the configure event
    :type serial: `uint`
    """
    self._marshal(4, serial)


@ZxdgSurfaceV6.event("u", [None])
def configure(self, serial):
    """Suggest a surface change

    The configure event marks the end of a configure sequence. A configure
    sequence is a set of one or more events configuring the state of the
    :class:`~pywayland.protocol.xdg_shell.XdgSurface`, including the final
    :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` event.

    Where applicable, :class:`~pywayland.protocol.xdg_shell.XdgSurface` surface
    roles will during a configure sequence extend this event as a latched state
    sent as events before the :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` event. Such events
    should be considered to make up a set of atomically applied configuration
    states, where the :func:`XdgSurface.configure()
    <pywayland.protocol.xdg_shell.XdgSurface..configure>` commits the
    accumulated state.

    Clients should arrange their surface for the new states, and then send an
    ack_configure request with the serial sent in this configure event at some
    point before committing the new surface.

    If the client receives multiple configure events before it can respond to
    one, it is free to discard all but the last event it received.

    :param serial:
        serial of the configure event
    :type serial: `uint`
    """
    self._post_event(0, serial)


ZxdgSurfaceV6._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
# Copyright © 2015, 2016 Jan Arne Petersen
# Copyright © 2017, 2018 Red Hat, Inc.
# Copyright © 2018       Purism SPC
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from pywayland.interface import Interface
from ..wayland import WlSurface

import enum


class ZwpTextInputV3(Interface):
    """Text input

    The :class:`ZwpTextInputV3` interface represents text input and input
    methods associated with a seat. It provides enter/leave events to follow
    the text input focus for a seat.

    Requests are used to enable/disable the text-input object and set state
    information like surrounding and selected text or the content type. The
    information about the entered text is sent to the text-input object via the
    preedit_string and commit_string events.

    Text is valid UTF-8 encoded, indices and lengths are in bytes. Indices must
    not point to middle bytes inside a code point: they must either point to
    the first byte of a code point or to the end of the buffer. Lengths must be
    measured between two valid indices.

    Focus moving throughout surfaces will result in the emission of
    :func:`ZwpTextInputV3.enter()` and :func:`ZwpTextInputV3.leave()` events.
    The focused surface must commit :func:`ZwpTextInputV3.enable()` and
    :func:`ZwpTextInputV3.disable()` requests as the keyboard focus moves
    across editable and non-editable elements of the UI. Those two requests are
    not expected to be paired with each other, the compositor must be able to
    handle consecutive series of the same request.

    State is sent by the state requests (set_surrounding_text, set_content_type
    and set_cursor_rectangle) and a commit request. After an enter event or
    disable request all state information is invalidated and needs to be resent
    by the client.
    """
    name = "zwp_text_input_v3"
    version = 1

    change_cause = enum.Enum("change_cause", {
        "input_method": 0,
        "other": 1,
    })

    content_hint = enum.Enum("content_hint", {
        "none": 0x0,
        "completion": 0x1,
        "spellcheck": 0x2,
        "auto_capitalization": 0x4,
        "lowercase": 0x8,
        "uppercase": 0x10,
        "titlecase": 0x20,
        "hidden_text": 0x40,
        "sensitive_data": 0x80,
        "latin": 0x100,
        "multiline": 0x200,
    })

    content_purpose = enum.Enum("content_purpose", {
        "normal": 0,
        "alpha": 1,
        "digits": 2,
        "number": 3,
        "phone": 4,
        "url": 5,
        "email": 6,
        "name": 7,
        "password": 8,
        "pin": 9,
        "date": 10,
        "time": 11,
        "datetime": 12,
        "terminal": 13,
    })


@ZwpTextInputV3.request("", [])
def destroy(self):
    """Destroy the wp_text_input

    Destroy the wp_text_input object. Also disables all surfaces enabled
    through this wp_text_input object.
    """
    self._marshal(0)
    self._destroy()


@ZwpTextInputV3.request("", [])
def enable(self):
    """Request text input to be enabled

    Requests text input on the surface previously obtained from the enter
    event.

    This request must be issued every time the active text input changes to a
    new one, including within the current surface. Use
    :func:`ZwpTextInputV3.disable()` when there is no longer any input focus on
    the current surface.

    This request resets all state associated with previous enable, disable,
    set_surrounding_text, set_text_change_cause, set_content_type, and
    set_cursor_rectangle requests, as well as the state associated with
    preedit_string, commit_string, and delete_surrounding_text events.

    The set_surrounding_text, set_content_type and set_cursor_rectangle
    requests must follow if the text input supports the necessary
    functionality.

    State set with this request is double-buffered. It will get applied on the
    next :func:`ZwpTextInputV3.commit()` request, and stay valid until the next
    committed enable or disable request.

    The changes must be applied by the compositor after issuing a
    :func:`ZwpTextInputV3.commit()` request.
    """
    self._marshal(1)


@ZwpTextInputV3.request("", [])
def disable(self):
    """Disable text input on a surface

    Explicitly disable text input on the current surface (typically when there
    is no focus on any text entry inside the surface).

    State set with this request is double-buffered. It will get applied on the
    next :func:`ZwpTextInputV3.commit()` request.
    """
    self._marshal(2)


@ZwpTextInputV3.request("sii", [None, None, None])
def set_surrounding_text(self, text, cursor, anchor):
    """Sets the surrounding text

    Sets the surrounding plain text around the input, excluding the preedit
    text.

    The client should notify the compositor of any changes in any of the values
    carried with this request, including changes caused by handling incoming
    text-input events as well as changes caused by other mechanisms like
    keyboard typing.

    If the client is unaware of the text around the cursor, it should not issue
    this request, to signify lack of support to the compositor.

    Text is UTF-8 encoded, and should include the cursor position, the complete
    selection and additional characters before and after them. There is a
    maximum length of wayland messages, so text can not be longer than 4000
    bytes.

    Cursor is the byte offset of the cursor within text buffer.

    Anchor is the byte offset of the selection anchor within text buffer. If
    there is no selected text, anchor is the same as cursor.

    If any preedit text is present, it is replaced with a cursor for the
    purpose of this event.

    Values set with this request are double-buffered. They will get applied on
    the next :func:`ZwpTextInputV3.commit()` request, and stay valid until the
    next committed enable or disable request.

    The initial state for affected fields is empty, meaning that the text input
    does not support sending surrounding text. If the empty values get applied,
    subsequent attempts to change them may have no effect.

    :param text:
    :type text: `string`
    :param cursor:
    :type cursor: `int`
    :param anchor:
    :type anchor: `int`
    """
    self._marshal(3, text, cursor, anchor)


@ZwpTextInputV3.request("u", [None])
def set_text_change_cause(self, cause):
    """Indicates the cause of surrounding text change

    Tells the compositor why the text surrounding the cursor changed.

    Whenever the client detects an external change in text, cursor, or anchor
    posision, it must issue this request to the compositor. This request is
    intended to give the input method a chance to update the preedit text in an
    appropriate way, e.g. by removing it when the user starts typing with a
    keyboard.

    cause describes the source of the change.

    The value set with this request is double-buffered. It must be applied and
    reset to initial at the next :func:`ZwpTextInputV3.commit()` request.

    The initial value of cause is input_method.

    :param cause:
    :type cause: `uint`
    """
    self._marshal(4, cause)


@ZwpTextInputV3.request("uu", [None, None])
def set_content_type(self, hint, purpose):
    """Set content purpose and hint

    Sets the content purpose and content hint. While the purpose is the basic
    purpose of an input field, the hint flags allow to modify some of the
    behavior.

    Values set with this request are double-buffered. They will get applied on
    the next :func:`ZwpTextInputV3.commit()` request. Subsequent attempts to
    update them may have no effect. The values remain valid until the next
    committed enable or disable request.

    The initial value for hint is none, and the initial value for purpose is
    normal.

    :param hint:
    :type hint: `uint`
    :param purpose:
    :type purpose: `uint`
    """
    self._marshal(5, hint, purpose)


@ZwpTextInputV3.request("iiii", [None, None, None, None])
def set_cursor_rectangle(self, x, y, width, height):
    """Set cursor position

    Marks an area around the cursor as a x, y, width, height rectangle in
    surface local coordinates.

    Allows the compositor to put a window with word suggestions near the
    cursor, without obstructing the text being input.

    If the client is unaware of the position of edited text, it should not
    issue this request, to signify lack of support to the compositor.

    Values set with this request are double-buffered. They will get applied on
    the next :func:`ZwpTextInputV3.commit()` request, and stay valid until the
    next committed enable or disable request.

    The initial values describing a cursor rectangle are empty. That means the
    text input does not support describing the cursor area. If the empty values
    get applied, subsequent attempts to change them may have no effect.

    :param x:
    :type x: `int`
    :param y:
    :type y: `int`
    :param width:
    :type width: `int`
    :param height:
    :type height: `int`
    """
    self._marshal(6, x, y, width, height)


@ZwpTextInputV3.request("", [])
def commit(self):
    """Commit state

    Atomically applies state changes recently sent to the compositor.

    The commit request establishes and updates the state of the client, and
    must be issued after any changes to apply them.

    Text input state (enabled status, content purpose, content hint,
    surrounding text and change cause, cursor rectangle) is conceptually
    double-buffered within the context of a text input, i.e. between a
    committed enable request and the following committed enable or disable
    request.

    Protocol requests modify the pending state, as opposed to the current state
    in use by the input method. A commit request atomically applies all pending
    state, replacing the current state. After commit, the new pending state is
    as documented for each related request.

    Requests are applied in the order of arrival.

    Neither current nor pending state are modified unless noted otherwise.

    The compositor must count the number of commit requests coming from each
    :class:`ZwpTextInputV3` object and use the count as the serial in done
    events.
    """
    self._marshal(7)


@ZwpTextInputV3.event("o", [WlSurface])
def enter(self, surface):
    """Enter event

    Notification that this seat's text-input focus is on a certain surface.

    When the seat has the keyboard capability the text-input focus follows the
    keyboard focus. This event sets the current surface for the text-input
    object.

    :param surface:
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    """
    self._post_event(0, surface)


@ZwpTextInputV3.event("o", [WlSurface])
def leave(self, surface):
    """Leave event

    Notification that this seat's text-input focus is no longer on a certain
    surface. The client should reset any preedit string previously set.

    The leave notification clears the current surface. It is sent before the
    enter notification for the new focus.

    When the seat has the keyboard capability the text-input focus follows the
    keyboard focus.

    :param surface:
    :type surface: :class:`~pywayland.protocol.wayland.WlSurface`
    """
    self._post_event(1, surface)


@ZwpTextInputV3.event("?sii", [None, None, None])
def preedit_string(self, text, cursor_begin, cursor_end):
    """Pre-edit

    Notify when a new composing text (pre-edit) should be set at the current
    cursor position. Any previously set composing text must be removed. Any
    previously existing selected text must be removed.

    The argument text contains the pre-edit string buffer.

    The parameters cursor_begin and cursor_end are counted in bytes relative to
    the beginning of the submitted text buffer. Cursor should be hidden when
    both are equal to -1.

    They could be represented by the client as a line if both values are the
    same, or as a text highlight otherwise.

    Values set with this event are double-buffered. They must be applied and
    reset to initial on the next :func:`ZwpTextInputV3.done()` event.

    The initial value of text is an empty string, and cursor_begin, cursor_end
    and cursor_hidden are all 0.

    :param text:
    :type text: `string` or `None`
    :param cursor_begin:
    :type cursor_begin: `int`
    :param cursor_end:
    :type cursor_end: `int`
    """
    self._post_event(2, text, cursor_begin, cursor_end)


@ZwpTextInputV3.event("?s", [None])
def commit_string(self, text):
    """Text commit

    Notify when text should be inserted into the editor widget. The text to
    commit could be either just a single character after a key press or the
    result of some composing (pre-edit).

    Values set with this event are double-buffered. They must be applied and
    reset to initial on the next :func:`ZwpTextInputV3.done()` event.

    The initial value of text is an empty string.

    :param text:
    :type text: `string` or `None`
    """
    self._post_event(3, text)


@ZwpTextInputV3.event("uu", [None, None])
def delete_surrounding_text(self, before_length, after_length):
    """Delete surrounding text

    Notify when the text around the current cursor position should be deleted.

    Before_length and after_length are the number of bytes before and after the
    current cursor index (excluding the selection) to delete.

    If a preedit text is present, in effect before_length is counted from the
    beginning of it, and after_length from its end (see done event sequence).

    Values set with this event are double-buffered. They must be applied and
    reset to initial on the next :func:`ZwpTextInputV3.done()` event.

    The initial values of both before_length and after_length are 0.

    :param before_length:
        length of text before current cursor position
    :type before_length: `uint`
    :param after_length:
        length of text after current cursor position
    :type after_length: `uint`
    """
    self._post_event(4, before_length, after_length)


@ZwpTextInputV3.event("u", [None])
def done(self, serial):
    """Apply changes

    Instruct the application to apply changes to state requested by the
    preedit_string, commit_string and delete_surrounding_text events. The state
    relating to these events is double-buffered, and each one modifies the
    pending state. This event replaces the current state with the pending
    state.

    The application must proceed by evaluating the changes in the following
    order:

    1. Replace existing preedit string with the cursor. 2. Delete requested
    surrounding text. 3. Insert commit string with the cursor at its end. 4.
    Calculate surrounding text to send. 5. Insert new preedit text in cursor
    position. 6. Place cursor inside preedit text.

    The serial number reflects the last state of the :class:`ZwpTextInputV3`
    object known to the compositor. The value of the serial argument must be
    equal to the number of commit requests already issued on that object. When
    the client receives a done event with a serial different than the number of
    past commit requests, it must proceed as normal, except it should not
    change the current state of the :class:`ZwpTextInputV3` object.

    :param serial:
    :type serial: `uint`
    """
    self._post_event(5, serial)


ZwpTextInputV3._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_buffer import WlBuffer
from .wl_callback import WlCallback
from .wl_output import WlOutput
from .wl_region import WlRegion

import enum


class WlSurface(Interface):
    """An onscreen surface

    A surface is a rectangular area that is displayed on the screen. It has a
    location, size and pixel contents.

    The size of a surface (and relative positions on it) is described in
    surface-local coordinates, which may differ from the buffer coordinates of
    the pixel content, in case a buffer_transform or a buffer_scale is used.

    A surface without a "role" is fairly useless: a compositor does not know
    where, when or how to present it. The role is the purpose of a
    :class:`WlSurface`. Examples of roles are a cursor for a pointer (as set by
    :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer..set_cursor>`), a drag icon
    (:func:`WlDataDevice.start_drag()
    <pywayland.protocol.wayland.WlDataDevice..start_drag>`), a sub-surface
    (:func:`WlSubcompositor.get_subsurface()
    <pywayland.protocol.wayland.WlSubcompositor..get_subsurface>`), and a
    window as defined by a shell protocol (e.g.
    :func:`WlShell.get_shell_surface()
    <pywayland.protocol.wayland.WlShell..get_shell_surface>`).

    A surface can have only one role at a time. Initially a :class:`WlSurface`
    does not have a role. Once a :class:`WlSurface` is given a role, it is set
    permanently for the whole lifetime of the :class:`WlSurface` object. Giving
    the current role again is allowed, unless explicitly forbidden by the
    relevant interface specification.

    Surface roles are given by requests in other interfaces such as
    :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer..set_cursor>`. The request should
    explicitly mention that this request gives a role to a :class:`WlSurface`.
    Often, this request also creates a new protocol object that represents the
    role and adds additional functionality to :class:`WlSurface`. When a client
    wants to destroy a :class:`WlSurface`, they must destroy this 'role object'
    before the :class:`WlSurface`.

    Destroying the role object does not remove the role from the
    :class:`WlSurface`, but it may stop the :class:`WlSurface` from "playing
    the role". For instance, if a
    :class:`~pywayland.protocol.wayland.WlSubsurface` object is destroyed, the
    :class:`WlSurface` it was created for will be unmapped and forget its
    position and z-order. It is allowed to create a
    :class:`~pywayland.protocol.wayland.WlSubsurface` for the same
    :class:`WlSurface` again, but it is not allowed to use the
    :class:`WlSurface` as a cursor (cursor is a different role than sub-
    surface, and role switching is not allowed).
    """
    name = "wl_surface"
    version = 4

    error = enum.Enum("error", {
        "invalid_scale": 0,
        "invalid_transform": 1,
    })


@WlSurface.request("", [])
def destroy(self):
    """Delete surface

    Deletes the surface and invalidates its object ID.
    """
    self._marshal(0)
    self._destroy()


@WlSurface.request("?oii", [WlBuffer, None, None])
def attach(self, buffer, x, y):
    """Set the surface contents

    Set a buffer as the content of this surface.

    The new size of the surface is calculated based on the buffer size
    transformed by the inverse buffer_transform and the inverse buffer_scale.
    This means that the supplied buffer must be an integer multiple of the
    buffer_scale.

    The x and y arguments specify the location of the new pending buffer's
    upper left corner, relative to the current buffer's upper left corner, in
    surface-local coordinates. In other words, the x and y, combined with the
    new surface size define in which directions the surface's size changes.

    Surface contents are double-buffered state, see :func:`WlSurface.commit()`.

    The initial surface contents are void; there is no content.
    :func:`WlSurface.attach()` assigns the given
    :class:`~pywayland.protocol.wayland.WlBuffer` as the pending
    :class:`~pywayland.protocol.wayland.WlBuffer`. :func:`WlSurface.commit()`
    makes the pending :class:`~pywayland.protocol.wayland.WlBuffer` the new
    surface contents, and the size of the surface becomes the size calculated
    from the :class:`~pywayland.protocol.wayland.WlBuffer`, as described above.
    After commit, there is no pending buffer until the next attach.

    Committing a pending :class:`~pywayland.protocol.wayland.WlBuffer` allows
    the compositor to read the pixels in the
    :class:`~pywayland.protocol.wayland.WlBuffer`. The compositor may access
    the pixels at any time after the :func:`WlSurface.commit()` request. When
    the compositor will not access the pixels anymore, it will send the
    :func:`WlBuffer.release() <pywayland.protocol.wayland.WlBuffer..release>`
    event. Only after receiving :func:`WlBuffer.release()
    <pywayland.protocol.wayland.WlBuffer..release>`, the client may reuse the
    :class:`~pywayland.protocol.wayland.WlBuffer`. A
    :class:`~pywayland.protocol.wayland.WlBuffer` that has been attached and
    then replaced by another attach instead of committed will not receive a
    release event, and is not used by the compositor.

    Destroying the :class:`~pywayland.protocol.wayland.WlBuffer` after
    :func:`WlBuffer.release() <pywayland.protocol.wayland.WlBuffer..release>`
    does not change the surface contents. However, if the client destroys the
    :class:`~pywayland.protocol.wayland.WlBuffer` before receiving the
    :func:`WlBuffer.release() <pywayland.protocol.wayland.WlBuffer..release>`
    event, the surface contents become undefined immediately.

    If :func:`WlSurface.attach()` is sent with a NULL
    :class:`~pywayland.protocol.wayland.WlBuffer`, the following
    :func:`WlSurface.commit()` will remove the surface content.

    :param buffer:
        buffer of surface contents
    :type buffer: :class:`~pywayland.protocol.wayland.WlBuffer` or `None`
    :param x:
        surface-local x coordinate
    :type x: `int`
    :param y:
        surface-local y coordinate
    :type y: `int`
    """
    self._marshal(1, buffer, x, y)


@WlSurface.request("iiii", [None, None, None, None])
def damage(self, x, y, width, height):
    """Mark part of the surface damaged

    This request is used to describe the regions where the pending buffer is
    different from the current surface contents, and where the surface
    therefore needs to be repainted. The compositor ignores the parts of the
    damage that fall outside of the surface.

    Damage is double-buffered state, see :func:`WlSurface.commit()`.

    The damage rectangle is specified in surface-local coordinates, where x and
    y specify the upper left corner of the damage rectangle.

    The initial value for pending damage is empty: no damage.
    :func:`WlSurface.damage()` adds pending damage: the new pending damage is
    the union of old pending damage and the given rectangle.

    :func:`WlSurface.commit()` assigns pending damage as the current damage,
    and clears pending damage. The server will clear the current damage as it
    repaints the surface.

    Note! New clients should not use this request. Instead damage can be posted
    with :func:`WlSurface.damage_buffer()` which uses buffer coordinates
    instead of surface coordinates.

    :param x:
        surface-local x coordinate
    :type x: `int`
    :param y:
        surface-local y coordinate
    :type y: `int`
    :param width:
        width of damage rectangle
    :type width: `int`
    :param height:
        height of damage rectangle
    :type height: `int`
    """
    self._marshal(2, x, y, width, height)


@WlSurface.request("n", [WlCallback])
def frame(self):
    """Request a frame throttling hint

    Request a notification when it is a good time to start drawing a new frame,
    by creating a frame callback. This is useful for throttling redrawing
    operations, and driving animations.

    When a client is animating on a :class:`WlSurface`, it can use the 'frame'
    request to get notified when it is a good time to draw and commit the next
    frame of animation. If the client commits an update earlier than that, it
    is likely that some updates will not make it to the display, and the client
    is wasting resources by drawing too often.

    The frame request will take effect on the next :func:`WlSurface.commit()`.
    The notification will only be posted for one frame unless requested again.
    For a :class:`WlSurface`, the notifications are posted in the order the
    frame requests were committed.

    The server must send the notifications so that a client will not send
    excessive updates, while still allowing the highest possible update rate
    for clients that wait for the reply before drawing again. The server should
    give some time for the client to draw and commit after sending the frame
    callback events to let it hit the next output refresh.

    A server should avoid signaling the frame callbacks if the surface is not
    visible in any way, e.g. the surface is off-screen, or completely obscured
    by other opaque surfaces.

    The object returned by this request will be destroyed by the compositor
    after the callback is fired and as such the client must not attempt to use
    it after that point.

    The callback_data passed in the callback is the current time, in
    milliseconds, with an undefined base.

    :returns:
        :class:`~pywayland.protocol.wayland.WlCallback` -- callback object for
        the frame request
    """
    callback = self._marshal_constructor(3, WlCallback)
    return callback


@WlSurface.request("?o", [WlRegion])
def set_opaque_region(self, region):
    """Set opaque region

    This request sets the region of the surface that contains opaque content.

    The opaque region is an optimization hint for the compositor that lets it
    optimize the redrawing of content behind opaque regions.  Setting an opaque
    region is not required for correct behaviour, but marking transparent
    content as opaque will result in repaint artifacts.

    The opaque region is specified in surface-local coordinates.

    The compositor ignores the parts of the opaque region that fall outside of
    the surface.

    Opaque region is double-buffered state, see :func:`WlSurface.commit()`.

    :func:`WlSurface.set_opaque_region()` changes the pending opaque region.
    :func:`WlSurface.commit()` copies the pending region to the current region.
    Otherwise, the pending and current regions are never changed.

    The initial value for an opaque region is empty. Setting the pending opaque
    region has copy semantics, and the
    :class:`~pywayland.protocol.wayland.WlRegion` object can be destroyed
    immediately. A NULL :class:`~pywayland.protocol.wayland.WlRegion` causes
    the pending opaque region to be set to empty.

    :param region:
        opaque region of the surface
    :type region: :class:`~pywayland.protocol.wayland.WlRegion` or `None`
    """
    self._marshal(4, region)


@WlSurface.request("?o", [WlRegion])
def set_input_region(self, region):
    """Set input region

    This request sets the region of the surface that can receive pointer and
    touch events.

    Input events happening outside of this region will try the next surface in
    the server surface stack. The compositor ignores the parts of the input
    region that fall outside of the surface.

    The input region is specified in surface-local coordinates.

    Input region is double-buffered state, see :func:`WlSurface.commit()`.

    :func:`WlSurface.set_input_region()` changes the pending input region.
    :func:`WlSurface.commit()` copies the pending region to the current region.
    Otherwise the pending and current regions are never changed, except cursor
    and icon surfaces are special cases, see :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer..set_cursor>` and
    :func:`WlDataDevice.start_drag()
    <pywayland.protocol.wayland.WlDataDevice..start_drag>`.

    The initial value for an input region is infinite. That means the whole
    surface will accept input. Setting the pending input region has copy
    semantics, and the :class:`~pywayland.protocol.wayland.WlRegion` object can
    be destroyed immediately. A NULL
    :class:`~pywayland.protocol.wayland.WlRegion` causes the input region to be
    set to infinite.

    :param region:
        input region of the surface
    :type region: :class:`~pywayland.protocol.wayland.WlRegion` or `None`
    """
    self._marshal(5, region)


@WlSurface.request("", [])
def commit(self):
    """Commit pending surface state

    Surface state (input, opaque, and damage regions, attached buffers, etc.)
    is double-buffered. Protocol requests modify the pending state, as opposed
    to the current state in use by the compositor. A commit request atomically
    applies all pending state, replacing the current state. After commit, the
    new pending state is as documented for each related request.

    On commit, a pending :class:`~pywayland.protocol.wayland.WlBuffer` is
    applied first, and all other state second. This means that all coordinates
    in double-buffered state are relative to the new
    :class:`~pywayland.protocol.wayland.WlBuffer` coming into use, except for
    :func:`WlSurface.attach()` itself. If there is no pending
    :class:`~pywayland.protocol.wayland.WlBuffer`, the coordinates are relative
    to the current surface contents.

    All requests that need a commit to become effective are documented to
    affect double-buffered state.

    Other interfaces may add further double-buffered surface state.
    """
    self._marshal(6)


@WlSurface.request("2i", [None])
def set_buffer_transform(self, transform):
    """Sets the buffer transformation

    This request sets an optional transformation on how the compositor
    interprets the contents of the buffer attached to the surface. The accepted
    values for the transform parameter are the values for
    :func:`WlOutput.transform()
    <pywayland.protocol.wayland.WlOutput..transform>`.

    Buffer transform is double-buffered state, see :func:`WlSurface.commit()`.

    A newly created surface has its buffer transformation set to normal.

    :func:`WlSurface.set_buffer_transform()` changes the pending buffer
    transformation. :func:`WlSurface.commit()` copies the pending buffer
    transformation to the current one. Otherwise, the pending and current
    values are never changed.

    The purpose of this request is to allow clients to render content according
    to the output transform, thus permitting the compositor to use certain
    optimizations even if the display is rotated. Using hardware overlays and
    scanning out a client buffer for fullscreen surfaces are examples of such
    optimizations. Those optimizations are highly dependent on the compositor
    implementation, so the use of this request should be considered on a case-
    by-case basis.

    Note that if the transform value includes 90 or 270 degree rotation, the
    width of the buffer will become the surface height and the height of the
    buffer will become the surface width.

    If transform is not one of the values from the :func:`WlOutput.transform()
    <pywayland.protocol.wayland.WlOutput..transform>` enum the
    invalid_transform protocol error is raised.

    :param transform:
        transform for interpreting buffer contents
    :type transform: `int`
    """
    self._marshal(7, transform)


@WlSurface.request("3i", [None])
def set_buffer_scale(self, scale):
    """Sets the buffer scaling factor

    This request sets an optional scaling factor on how the compositor
    interprets the contents of the buffer attached to the window.

    Buffer scale is double-buffered state, see :func:`WlSurface.commit()`.

    A newly created surface has its buffer scale set to 1.

    :func:`WlSurface.set_buffer_scale()` changes the pending buffer scale.
    :func:`WlSurface.commit()` copies the pending buffer scale to the current
    one. Otherwise, the pending and current values are never changed.

    The purpose of this request is to allow clients to supply higher resolution
    buffer data for use on high resolution outputs. It is intended that you
    pick the same buffer scale as the scale of the output that the surface is
    displayed on. This means the compositor can avoid scaling when rendering
    the surface on that output.

    Note that if the scale is larger than 1, then you have to attach a buffer
    that is larger (by a factor of scale in each dimension) than the desired
    surface size.

    If scale is not positive the invalid_scale protocol error is raised.

    :param scale:
        positive scale for interpreting buffer contents
    :type scale: `int`
    """
    self._marshal(8, scale)


@WlSurface.request("4iiii", [None, None, None, None])
def damage_buffer(self, x, y, width, height):
    """Mark part of the surface damaged using buffer coordinates

    This request is used to describe the regions where the pending buffer is
    different from the current surface contents, and where the surface
    therefore needs to be repainted. The compositor ignores the parts of the
    damage that fall outside of the surface.

    Damage is double-buffered state, see :func:`WlSurface.commit()`.

    The damage rectangle is specified in buffer coordinates, where x and y
    specify the upper left corner of the damage rectangle.

    The initial value for pending damage is empty: no damage.
    :func:`WlSurface.damage_buffer()` adds pending damage: the new pending
    damage is the union of old pending damage and the given rectangle.

    :func:`WlSurface.commit()` assigns pending damage as the current damage,
    and clears pending damage. The server will clear the current damage as it
    repaints the surface.

    This request differs from :func:`WlSurface.damage()` in only one way - it
    takes damage in buffer coordinates instead of surface-local coordinates.
    While this generally is more intuitive than surface coordinates, it is
    especially desirable when using wp_viewport or when a drawing library (like
    EGL) is unaware of buffer scale and buffer transform.

    Note: Because buffer transformation changes and damage requests may be
    interleaved in the protocol stream, it is impossible to determine the
    actual mapping between surface and buffer damage until
    :func:`WlSurface.commit()` time. Therefore, compositors wishing to take
    both kinds of damage into account will have to accumulate damage from the
    two requests separately and only transform from one to the other after
    receiving the :func:`WlSurface.commit()`.

    :param x:
        buffer-local x coordinate
    :type x: `int`
    :param y:
        buffer-local y coordinate
    :type y: `int`
    :param width:
        width of damage rectangle
    :type width: `int`
    :param height:
        height of damage rectangle
    :type height: `int`
    """
    self._marshal(9, x, y, width, height)


@WlSurface.event("o", [WlOutput])
def enter(self, output):
    """Surface enters an output

    This is emitted whenever a surface's creation, movement, or resizing
    results in some part of it being within the scanout region of an output.

    Note that a surface may be overlapping with zero or more outputs.

    :param output:
        output entered by the surface
    :type output: :class:`~pywayland.protocol.wayland.WlOutput`
    """
    self._post_event(0, output)


@WlSurface.event("o", [WlOutput])
def leave(self, output):
    """Surface leaves an output

    This is emitted whenever a surface's creation, movement, or resizing
    results in it no longer having any part of it within the scanout region of
    an output.

    :param output:
        output left by the surface
    :type output: :class:`~pywayland.protocol.wayland.WlOutput`
    """
    self._post_event(1, output)


WlSurface._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import textwrap
from typing import NamedTuple, List  # noqa: F401


Attribute = NamedTuple("Attribute", [("name", str), ("required", bool)])

Child = NamedTuple("Child", [("name", str), ("class_", object), ("required", bool), ("allow_multiple", bool)])


class Element:
    attributes = []  # type: List[Attribute]
    children = []  # type: List[Child]
    pcdata = False

    def __init__(self, element, *args, **kwargs):
        for attr in self.attributes:
            obj = element.attrib.get(attr.name)

            if not attr and attr.required:
                raise ValueError()

            name = attr.name.replace('-', '_')
            setattr(self, name, obj)

        for child in self.children:
            if child.allow_multiple:
                obj = [child.class_(elem) for elem in element.findall(child.name)]
            else:
                obj = element.find(child.name)
                if obj is not None:
                    obj = child.class_(obj)

            if not obj and child.required:
                raise ValueError(child.name)

            # We need to replace dashes with underscores
            name = child.name.replace('-', '_')
            setattr(self, name, obj)

        if self.pcdata:
            text = element.text
            # We need to strip each line while keeping paragraph breaks
            if text:
                self.text = textwrap.dedent(text.expandtabs(8).rstrip().lstrip('\n'))
            else:
                self.text = None



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.interface import Interface
from .wl_data_device import WlDataDevice
from .wl_data_source import WlDataSource
from .wl_seat import WlSeat

import enum


class WlDataDeviceManager(Interface):
    """Data transfer interface

    The :class:`WlDataDeviceManager` is a singleton global object that provides
    access to inter-client data transfer mechanisms such as copy-and-paste and
    drag-and-drop.  These mechanisms are tied to a
    :class:`~pywayland.protocol.wayland.WlSeat` and this interface lets a
    client get a :class:`~pywayland.protocol.wayland.WlDataDevice`
    corresponding to a :class:`~pywayland.protocol.wayland.WlSeat`.

    Depending on the version bound, the objects created from the bound
    :class:`WlDataDeviceManager` object will have different requirements for
    functioning properly. See :func:`WlDataSource.set_actions()
    <pywayland.protocol.wayland.WlDataSource..set_actions>`,
    :func:`WlDataOffer.accept()
    <pywayland.protocol.wayland.WlDataOffer..accept>` and
    :func:`WlDataOffer.finish()
    <pywayland.protocol.wayland.WlDataOffer..finish>` for details.
    """
    name = "wl_data_device_manager"
    version = 3

    dnd_action = enum.Enum("dnd_action", {
        "none": 0,
        "copy": 1,
        "move": 2,
        "ask": 4,
    })


@WlDataDeviceManager.request("n", [WlDataSource])
def create_data_source(self):
    """Create a new data source

    Create a new data source.

    :returns:
        :class:`~pywayland.protocol.wayland.WlDataSource` -- data source to
        create
    """
    id = self._marshal_constructor(0, WlDataSource)
    return id


@WlDataDeviceManager.request("no", [WlDataDevice, WlSeat])
def get_data_device(self, seat):
    """Create a new data device

    Create a new data device for a given seat.

    :param seat:
        seat associated with the data device
    :type seat: :class:`~pywayland.protocol.wayland.WlSeat`
    :returns:
        :class:`~pywayland.protocol.wayland.WlDataDevice` -- data device to
        create
    """
    id = self._marshal_constructor(1, WlDataDevice, seat)
    return id


WlDataDeviceManager._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .description import Description
from .entry import Entry
from .element import Element, Attribute, Child


class Enum(Element):
    """Scanner for enum objects

    Required attributes: `name` and `since`

    Child elements: `description` and `entry`
    """

    attributes = [
        Attribute('name', True),
        Attribute('since', False),
        Attribute('bitfield', False)
    ]

    children = [
        Child('description', Description, False, False),
        Child('entry', Entry, False, True)
    ]

    def output(self, printer):
        """Generate the output for the enum to the printer"""
        name = self.name if self.name != "version" else "version_"
        printer('{0} = enum.Enum("{0}", {{'.format(name))
        with printer.indented():
            for entry in self.entry:
                entry.output(printer)
        printer('})')



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface
from .wl_core import WlCore
from .wl_requests import WlRequests


class WlEvents(Interface):
    """Events object

    The interface object with the different types of events.
    """
    name = "wl_events"
    version = 2


@WlEvents.event("niuh", [WlRequests, None, None, None])
def send_event(self, id, the_int, the_uint, the_fd):
    """Send the data

    Request for data from the client.  Send the data as the specified mime type
    over the passed file descriptor, then close it.

    :param id:
    :type id: :class:`~pywayland.protocol.scanner_test.WlRequests`
    :param the_int:
    :type the_int: `int`
    :param the_uint:
        the arg summary
    :type the_uint: `uint`
    :param the_fd:
    :type the_fd: `fd`
    """
    self._post_event(0, id, the_int, the_uint, the_fd)


@WlEvents.event("", [])
def no_args(self):
    """Event with no args

    An event method that does not have any arguments.
    """
    self._post_event(1)


@WlEvents.event("n", [WlCore])
def create_id(self, id):
    """Create an id

    With a description

    :param id:
    :type id: :class:`~pywayland.protocol.scanner_test.WlCore`
    """
    self._post_event(2, id)


@WlEvents.event("n", [WlCore])
def create_id2(self, id):
    """Create an id without a description

    :param id:
    :type id: :class:`~pywayland.protocol.scanner_test.WlCore`
    """
    self._post_event(3, id)


@WlEvents.event("?s", [None])
def allow_null_event(self, null_string):
    """A event with an allowed null argument

    An event where one of the arguments is allowed to be null.

    :param null_string:
    :type null_string: `string` or `None`
    """
    self._post_event(4, null_string)


@WlEvents.event("n?o", [WlRequests, WlCore])
def make_import(self, id, object):
    """Event that causes an import

    An event method that causes an imoprt of other interfaces

    :param id:
    :type id: :class:`~pywayland.protocol.scanner_test.WlRequests`
    :param object:
    :type object: :class:`~pywayland.protocol.scanner_test.WlCore` or `None`
    """
    self._post_event(5, id, object)


@WlEvents.event("2", [])
def versioned(self):
    """A versioned event

    An event that is versioned.
    """
    self._post_event(6)


WlEvents._gen_c()



# -*- coding: utf-8 -*-
# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.interface import Interface
from .wl_core import WlCore
from .wl_events import WlEvents


class WlRequests(Interface):
    """Request object

    The interface object with the different types of requests.
    """
    name = "wl_requests"
    version = 2


@WlRequests.request("niuh", [WlCore, None, None, None])
def make_request(self, the_int, the_uint, the_fd):
    """A request

    The request asks the server for an event.

    :param the_int:
    :type the_int: `int`
    :param the_uint:
        the arg summary
    :type the_uint: `uint`
    :param the_fd:
    :type the_fd: `fd`
    :returns:
        :class:`~pywayland.protocol.scanner_test.WlCore`
    """
    id = self._marshal_constructor(0, WlCore, the_int, the_uint, the_fd)
    return id


@WlRequests.request("", [])
def no_args(self):
    """Request with no args

    A request method that does not have any arguments.
    """
    self._marshal(1)


@WlRequests.request("n", [WlCore])
def create_id(self):
    """Create an id

    With a description

    :returns:
        :class:`~pywayland.protocol.scanner_test.WlCore`
    """
    id = self._marshal_constructor(2, WlCore)
    return id


@WlRequests.request("n", [WlCore])
def create_id2(self):
    """Create an id without a description

    :returns:
        :class:`~pywayland.protocol.scanner_test.WlCore`
    """
    id = self._marshal_constructor(3, WlCore)
    return id


@WlRequests.request("u?s", [None, None])
def allow_null(self, serial, mime_type):
    """Request that allows for null arguments

    A request where one of the arguments is allowed to be null.

    :param serial:
    :type serial: `uint`
    :param mime_type:
    :type mime_type: `string` or `None`
    """
    self._marshal(4, serial, mime_type)


@WlRequests.request("n?o", [WlEvents, WlCore])
def make_import(self, object):
    """Request that causes an import

    A request method that causes an imoprt of other interfaces, both as a
    new_id and as an object.

    :param object:
    :type object: :class:`~pywayland.protocol.scanner_test.WlCore` or `None`
    :returns:
        :class:`~pywayland.protocol.scanner_test.WlEvents`
    """
    id = self._marshal_constructor(5, WlEvents, object)
    return id


@WlRequests.request("2", [])
def versioned(self):
    """A versioned request

    A request that is versioned.
    """
    self._marshal(6)


@WlRequests.request("usun", [None, None, None, None])
def new_id_no_interface(self, name, interface, version):
    """Create a new id, but with no interface

    A method with an argument for a new_id, but with no corresponding interface
    (c.f. wl_registry.bind).

    :param name:
    :type name: `uint`
    :param interface: Interface name
    :type interface: `string`
    :param version: Interface version
    :type version: `int`
    :returns:
        :class:`pywayland.client.proxy.Proxy` of specified Interface
    """
    id = self._marshal_constructor(7, interface, name, interface.name, version)
    return id


WlRequests._gen_c()



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .element import Element, Attribute, Child
from .description import Description


class Entry(Element):
    """Scanner for enum entries

    Required attributes: `name` and `value`

    Optional attributes: `summary` and `since`

    Child elements: `description`
    """
    attributes = [
        Attribute('name', True),
        Attribute('value', True),
        Attribute('summary', False),
        Attribute('since', False)
    ]
    children = [
        Child('description', Description, False, False)
    ]

    def output(self, printer):
        """Generate the output for the entry in the enum"""
        printer('"{}": {},'.format(self.name, self.value))
